@@ -21,6 +21,12 @@ import dbus
 import logging
 import os
 import re
+import traceback
+
+from subprocess import (
+    Popen,
+    PIPE,
+    )
 
 from gi.repository import GObject
 
@@ -29,7 +35,11 @@ from softwarecenter.utils import (sources_filename_from_ppa_entry,
                                   obfuscate_private_ppa_details,
                                   utf8,
                                   )
-from softwarecenter.enums import TransactionTypes, PURCHASE_TRANSACTION_ID
+from softwarecenter.enums import (
+    TransactionTypes,
+    PURCHASE_TRANSACTION_ID,
+    )
+from softwarecenter.paths import APPORT_RECOVERABLE_ERROR
 
 from aptdaemon import client
 from aptdaemon import enums
@@ -77,6 +87,7 @@ class FakePurchaseTransaction(object):
 
 
 class AptdaemonTransaction(BaseTransaction):
+
     def __init__(self, trans):
         self._trans = trans
 
@@ -204,8 +215,11 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
                                         (GObject.TYPE_PYOBJECT, bool,)),
                     }
 
+    LICENSE_KEY_SERVER = "ubuntu-production"
+
     def __init__(self):
         GObject.GObject.__init__(self)
+        InstallBackend.__init__(self)
 
         bus = get_dbus_bus()
         self.aptd_client = client.AptClient(bus=bus)
@@ -246,23 +260,25 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
 
     @inline_callbacks
     def fix_broken_depends(self):
+        trans = None
         try:
             trans = yield self.aptd_client.fix_broken_depends(defer=True)
             self.emit("transaction-started", "", "", trans.tid,
                 TransactionTypes.REPAIR)
             yield self._run_transaction(trans, None, None, None)
         except Exception as error:
-            self._on_trans_error(error)
+            self._on_trans_error(error, trans)
 
     @inline_callbacks
     def fix_incomplete_install(self):
+        trans = None
         try:
             trans = yield self.aptd_client.fix_incomplete_install(defer=True)
             self.emit("transaction-started", "", "", trans.tid,
                       TransactionTypes.REPAIR)
             yield self._run_transaction(trans, None, None, None)
         except Exception as error:
-            self._on_trans_error(error)
+            self._on_trans_error(error, trans)
 
     # FIXME: upgrade add-ons here
     @inline_callbacks
@@ -271,6 +287,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
         """ upgrade a single package """
         pkgname = app.pkgname
         appname = app.appname
+        trans = None
         try:
             trans = yield self.aptd_client.upgrade_packages([pkgname],
                                                             defer=True)
@@ -279,7 +296,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
             yield self._run_transaction(trans, pkgname, appname, iconname,
                 metadata)
         except Exception as error:
-            self._on_trans_error(error, pkgname)
+            self._on_trans_error(error, trans, pkgname)
 
 # broken
 #    @inline_callbacks
@@ -313,6 +330,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
         """ remove a single package """
         pkgname = app.pkgname
         appname = app.appname
+        trans = None
         try:
             trans = yield self.aptd_client.remove_packages([pkgname],
                                                            defer=True)
@@ -321,13 +339,13 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
             yield self._run_transaction(trans, pkgname, appname, iconname,
                 metadata)
         except Exception as error:
-            self._on_trans_error(error, pkgname)
+            self._on_trans_error(error, trans, pkgname)
 
     @inline_callbacks
     def remove_multiple(self, apps, iconnames, addons_install=[],
         addons_remove=[], metadatas=None):
         """ queue a list of packages for removal  """
-        if metadatas == None:
+        if metadatas is None:
             metadatas = []
             for item in apps:
                 metadatas.append(None)
@@ -340,6 +358,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
         """Install a single package from the archive
            If filename is given a local deb package is installed instead.
         """
+        trans = None
         pkgname = app.pkgname
         appname = app.appname
         # this will force aptdaemon to use the right archive suite on install
@@ -365,13 +384,13 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
             yield self._run_transaction(
                 trans, pkgname, appname, iconname, metadata)
         except Exception as error:
-            self._on_trans_error(error, pkgname)
+            self._on_trans_error(error, trans, pkgname)
 
     @inline_callbacks
     def install_multiple(self, apps, iconnames, addons_install=[],
         addons_remove=[], metadatas=None):
         """ queue a list of packages for install  """
-        if metadatas == None:
+        if metadatas is None:
             metadatas = []
             for item in apps:
                 metadatas.append(None)
@@ -384,6 +403,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
         """ install and remove add-ons """
         pkgname = app.pkgname
         appname = app.appname
+        trans = None
         try:
             install = addons_install
             remove = addons_remove
@@ -395,7 +415,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
                 TransactionTypes.APPLY)
             yield self._run_transaction(trans, pkgname, appname, iconname)
         except Exception as error:
-            self._on_trans_error(error)
+            self._on_trans_error(error, trans)
 
     @inline_callbacks
     def reload(self, sources_list=None, metadata=None):
@@ -406,6 +426,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
         # aptsources will just enable it instead of adding a extra
         # sources.list.d file (LP: #666956)
         d = apt_pkg.config.find_dir("Dir::Etc::sourceparts")
+        trans = None
         if (not sources_list or
             not os.path.exists(os.path.join(d, sources_list))):
             sources_list = ""
@@ -414,26 +435,28 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
                 sources_list=sources_list, defer=True)
             yield self._run_transaction(trans, None, None, None, metadata)
         except Exception as error:
-            self._on_trans_error(error)
+            self._on_trans_error(error, trans)
         # note that the cache re-open will happen via the connected
         # "transaction-finished" signal
 
     @inline_callbacks
     def enable_component(self, component):
         self._logger.debug("enable_component: %s" % component)
+        trans = None
         try:
             trans = yield self.aptd_client.enable_distro_component(component)
             # don't use self._run_transaction() here, to avoid sending uneeded
             # signals
             yield trans.run(defer=True)
         except Exception as error:
-            self._on_trans_error(error, component)
+            self._on_trans_error(error, trans, component)
             return_value(None)
         # now update the cache
         yield self.reload()
 
     @inline_callbacks
     def enable_channel(self, channelfile):
+        trans = None
         # read channel file and add all relevant lines
         for line in open(channelfile):
             line = line.strip()
@@ -461,12 +484,13 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
             keyid = keyid.split("/")[1]
         if not keyid.startswith("0x"):
             keyid = "0x%s" % keyid
+        trans = None
         try:
             trans = yield self.aptd_client.add_vendor_key_from_keyserver(
                 keyid, keyserver, defer=True)
             yield self._run_transaction(trans, None, None, None, metadata)
         except Exception as error:
-            self._on_trans_error(error)
+            self._on_trans_error(error, trans)
 
     @inline_callbacks
     def add_sources_list_entry(self, source_entry, sourcepart=None):
@@ -482,6 +506,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
 
         args = (entry.type, entry.uri, entry.dist, entry.comps,
                 "Added by software-center", sourcepart)
+        trans = None
         try:
             trans = yield self.aptd_client.add_repository(*args, defer=True)
             yield self._run_transaction(trans, None, None, None)
@@ -508,7 +533,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
         self._logger.debug(
             "adding license_key for pkg '%s' of len: %i" % (
                 pkgname, len(license_key)))
-
+        trans = None
         # HOME based license keys
         if license_key_path and license_key_path.startswith("~"):
             # check if its inside HOME and if so, just create it
@@ -527,9 +552,8 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
             # system-wide keys
             try:
                 self._logger.info("adding license key for '%s'" % pkgname)
-                server = "ubuntu-production"
                 trans = yield self.aptd_client.add_license_key(
-                    pkgname, license_key_oauth, server)
+                    pkgname, license_key_oauth, self.LICENSE_KEY_SERVER)
                 yield self._run_transaction(trans, None, None, None)
             except Exception as e:
                 self._logger.error("add_license_key: '%s'" % e)
@@ -586,7 +610,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
             'sc_add_repo_and_install_try': "1",
             'sc_add_repo_and_install_license_key': license_key or "",
             'sc_add_repo_and_install_license_key_path': license_key_path or "",
-            'sc_add_repo_and_install_license_key_token': \
+            'sc_add_repo_and_install_license_key_token':
                 json_oauth_token or "",
         }
 
@@ -691,7 +715,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
                 yield self._run_transaction(trans, app.pkgname, app.appname,
                                             "", metadata)
             except Exception as error:
-                self._on_trans_error(error, app.pkgname)
+                self._on_trans_error(error, trans, app.pkgname)
             # add license_key
             # FIXME: aptd fails if there is a license_key_path already
             #        but I wonder if we should ease that restriction
@@ -961,7 +985,7 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
                 yield trans.set_meta_data(defer=True, **metadata)
             yield trans.run(defer=True)
         except Exception as error:
-            self._on_trans_error(error, pkgname)
+            self._on_trans_error(error, trans, pkgname)
             # on error we need to clean the pending purchases
             self._clean_pending_purchases(pkgname)
         # on success the pending purchase is cleaned when the package
@@ -973,8 +997,9 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
         if pkgname and pkgname in self.pending_purchases:
             del self.pending_purchases[pkgname]
 
-    def _on_trans_error(self, error, pkgname=None):
-        self._logger.warn("_on_trans_error: %s", error)
+    def _on_trans_error(self, error, trans, pkgname=None):
+        self._logger.warn("_on_trans_error: '%r' '%s' '%s'" % (
+                error, trans, pkgname))
         # re-enable the action button again if anything went wrong
         result = TransactionFinishedResult(None, False)
         result.pkgname = pkgname
@@ -985,13 +1010,58 @@ class AptdaemonBackend(GObject.GObject, InstallBackend):
 
         self.emit("transaction-stopped", result)
         if isinstance(error, dbus.DBusException):
+            # ignore errors that the user knows about already (like
+            # that he entered a wrong password or that he does not
+            # care about (like no-reply)
             name = error.get_dbus_name()
             if name in ["org.freedesktop.PolicyKit.Error.NotAuthorized",
                         "org.freedesktop.DBus.Error.NoReply"]:
-                pass
-        else:
-            self._logger.exception("_on_trans_error")
-            #raise error
+                return
+            # we want to give some advice here to the user but also
+            # report this via apport
+            if name in ["org.freedesktop.PolicyKit.Error.Failed"]:
+                summary = _("Authentication Error")
+                text = _("Software can't be installed or removed because "
+                         "the authentication service is not available. "
+                         "(%s") % error
+                # send to apport for reporting
+                self._call_apport_recoverable_error(text, error)
+                # ... and display as a dialog
+                self.ui.error(None, summary, text)
+                return
+
+        # lintian errors are ignored and not send to apport_recoverable_error
+        if trans.error_code == enums.ERROR_INVALID_PACKAGE_FILE:
+            return
+
+        # show a apport recoverable error dialog to the user as we want
+        # to know about these issues
+        self._call_apport_recoverable_error(
+            _("There was an error submitting the transaction"), error)
+
+    def _call_apport_recoverable_error(self, text, error):
+        """Call apports recoverable_problem dialog """
+
+        # ensure we have a proper expection string in the report
+        if isinstance(error, Exception):
+            error = traceback.format_exc(error)
+
+        # mvo: I don't think we need to send "Package\0software-center",
+        #      apport should figure this out itself
+        data = "DialogBody\0%(text)s\0Traceback\0%(error)s" % {
+                'text': text,
+                'error': error,
+                }
+        # This will be quick as it just writes the report file. Then
+        # the report gets picked up asynchronously by a inotify watch
+        # and displayed to the user in a seperate process.
+        p = Popen(
+            [APPORT_RECOVERABLE_ERROR], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+
+        (stdout, stderr) = p.communicate(input=data)
+        if p.returncode != 0:
+            logging.warn("%s returned '%s' ('%s', '%s')" % (
+                    APPORT_RECOVERABLE_ERROR, p.returncode, stdout, stderr))
 
 
 if __name__ == "__main__":
