@@ -155,6 +155,7 @@ class UsefulnessCache(object):
 
 class Review(object):
     """A individual review object """
+
     def __init__(self, app):
         # a softwarecenter.db.database.Application object
         self.app = app
@@ -230,8 +231,31 @@ class ReviewLoader(GObject.GObject):
     __gsignals__ = {
         "refresh-review-stats-finished": (GObject.SIGNAL_RUN_LAST,
                                           GObject.TYPE_NONE,
+                                          # review-stats list
                                           (GObject.TYPE_PYOBJECT,),
-                                         ),
+                                          ),
+        "get-reviews-finished": (GObject.SIGNAL_RUN_LAST,
+                                 GObject.TYPE_NONE,
+                                 # application, reviewslist
+                                 (GObject.TYPE_PYOBJECT,
+                                  GObject.TYPE_PYOBJECT),
+                                 ),
+        "remove-review": (GObject.SIGNAL_RUN_LAST,
+                          GObject.TYPE_NONE,
+                          # app, the review
+                          (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT),
+                          ),
+        "replace-review": (GObject.SIGNAL_RUN_LAST,
+                           GObject.TYPE_NONE,
+                           # app, the review
+                           (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT),
+                           ),
+        "update-usefulness-votes": (GObject.SIGNAL_RUN_LAST,
+                                    GObject.TYPE_NONE,
+                                    # useful_votes
+                                    (GObject.TYPE_PYOBJECT, ),
+                                    ),
+
     }
 
     # cache the ReviewStats
@@ -275,7 +299,7 @@ class ReviewLoader(GObject.GObject):
                 return True
         return False
 
-    def get_reviews(self, application, callback, page=1, language=None,
+    def get_reviews(self, application, page=1, language=None,
                     sort=0, relaxed=False):
         """run callback f(app, review_list)
            with list of review objects for the given
@@ -300,7 +324,7 @@ class ReviewLoader(GObject.GObject):
         except ValueError:
             pass
 
-    def refresh_review_stats(self, callback):
+    def refresh_review_stats(self):
         """ get the review statists and call callback when its there """
         pass
 
@@ -477,7 +501,7 @@ class ReviewLoaderFake(ReviewLoader):
     def _random_summary(self):
         return random.choice(self.SUMMARIES)
 
-    def get_reviews(self, application, callback, page=1, language=None,
+    def get_reviews(self, application, page=1, language=None,
         sort=0, relaxed=False):
         if not application in self._review_stats_cache:
             self.get_review_stats(application)
@@ -498,7 +522,7 @@ class ReviewLoaderFake(ReviewLoader):
                 reviews.append(review)
             self._reviews_cache[application] = reviews
         reviews = self._reviews_cache[application]
-        callback(application, reviews)
+        self.emit("get-reviews-finished", application, reviews)
 
     def get_review_stats(self, application):
         if not application in self._review_stats_cache:
@@ -508,9 +532,9 @@ class ReviewLoaderFake(ReviewLoader):
             self._review_stats_cache[application] = stat
         return self._review_stats_cache[application]
 
-    def refresh_review_stats(self, callback):
+    def refresh_review_stats(self):
         review_stats = []
-        callback(review_stats)
+        self.emit("refresh-review-stats-finished", review_stats)
 
 
 class ReviewLoaderFortune(ReviewLoaderFake):
@@ -658,16 +682,16 @@ class ReviewLoaderNull(ReviewLoader):
         self._review_stats_cache = {}
         self._reviews_cache = {}
 
-    def get_reviews(self, application, callback, page=1, language=None,
+    def get_reviews(self, application, page=1, language=None,
         sort=0, relaxed=False):
-        callback(application, [])
+        self.emit("get-reviews-finished", application, [])
 
     def get_review_stats(self, application):
         pass
 
-    def refresh_review_stats(self, callback):
+    def refresh_review_stats(self):
         review_stats = []
-        callback(review_stats)
+        self.emit("refresh-review-stats-finished", review_stats)
 
 
 review_loader = None
@@ -696,11 +720,11 @@ def get_review_loader(cache, db=None):
     return review_loader
 
 if __name__ == "__main__":
-    def callback(app, reviews):
+    def callback(loader, app, reviews):
         print "app callback:"
         print app, reviews
 
-    def stats_callback(stats):
+    def stats_callback(loader, stats):
         print "stats callback:"
         print stats
 
@@ -720,8 +744,10 @@ if __name__ == "__main__":
         ReviewLoaderSpawningRNRClient
     )
     loader = ReviewLoaderSpawningRNRClient(cache, db)
-    print loader.refresh_review_stats(stats_callback)
-    print loader.get_reviews(app, callback)
+    loader.connect("refresh-review-stats-finished", stats_callback)
+    loader.connect("get-reviews-finished", callback)
+    loader.refresh_review_stats()
+    print loader.get_reviews(app)
 
     print "\n\n"
     print "default loader, press ctrl-c for next loader"
@@ -733,5 +759,5 @@ if __name__ == "__main__":
     app = Application("", "2vcard")
     loader = get_review_loader(cache, db)
     loader.refresh_review_stats(stats_callback)
-    loader.get_reviews(app, callback)
+    loader.get_reviews(app)
     main.run()
