@@ -69,10 +69,10 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         self.rnrclient._offline_mode = not network_state_is_connected()
 
     # reviews
-    def get_reviews(self, translated_app, callback, page=1,
+    def get_reviews(self, translated_app, page=1,
                     language=None, sort=0, relaxed=False):
-        """ public api, triggers fetching a review and calls callback
-            when its ready
+        """ public api, triggers fetching a review and emits
+            get-reviews-finished signal when its ready
         """
         # its fine to use the translated appname here, we only submit the
         # pkgname to the server
@@ -100,7 +100,7 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
                     origin = "lp-ppa-%s" % ppa.replace("/", "-")
             # if there is no origin, there is nothing to do
             if not origin:
-                callback(app, [])
+                self.emit("get-reviews-finished", app, [])
                 return
             distroseries = self.distro.get_codename()
         # run the command and add watcher
@@ -115,22 +115,21 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
               ]
         spawn_helper = SpawnHelper()
         spawn_helper.connect(
-            "data-available", self._on_reviews_helper_data, app, callback)
+            "data-available", self._on_reviews_helper_data, app)
         spawn_helper.run(cmd)
 
-    def _on_reviews_helper_data(self, spawn_helper, piston_reviews, app,
-        callback):
+    def _on_reviews_helper_data(self, spawn_helper, piston_reviews, app):
         # convert into our review objects
         reviews = []
         for r in piston_reviews:
             reviews.append(Review.from_piston_mini_client(r))
-        # add to our dicts and run callback
+        # add to our dicts and emit signal
         self._reviews[app] = reviews
-        callback(app, self._reviews[app])
+        self.emit("get-reviews-finished", app, self._reviews[app])
         return False
 
     # stats
-    def refresh_review_stats(self, callback):
+    def refresh_review_stats(self):
         """ public api, refresh the available statistics """
         try:
             mtime = os.path.getmtime(self.REVIEW_STATS_CACHE_FILE)
@@ -145,8 +144,7 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         #origin = "any"
         #distroseries = self.distro.get_codename()
         spawn_helper = SpawnHelper()
-        spawn_helper.connect("data-available", self._on_review_stats_data,
-            callback)
+        spawn_helper.connect("data-available", self._on_review_stats_data)
         if days_delta:
             spawn_helper.run_generic_piston_helper(
                 "RatingsAndReviewsAPI", "review_stats", days=days_delta)
@@ -154,8 +152,7 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
             spawn_helper.run_generic_piston_helper(
                 "RatingsAndReviewsAPI", "review_stats")
 
-    def _on_review_stats_data(self, spawn_helper, piston_review_stats,
-        callback):
+    def _on_review_stats_data(self, spawn_helper, piston_review_stats):
         """ process stdout from the helper """
         review_stats = self.REVIEW_STATS_CACHE
 
@@ -163,7 +160,7 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
             piston_review_stats):
             self.REVIEW_STATS_CACHE = {}
             self.save_review_stats_cache_file()
-            self.refresh_review_stats(callback)
+            self.refresh_review_stats()
             return
 
         # convert to the format that s-c uses
@@ -178,7 +175,6 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
             s.dampened_rating = calc_dr(s.rating_spread)
             review_stats[s.app] = s
         self.REVIEW_STATS_CACHE = review_stats
-        callback(review_stats)
         self.emit("refresh-review-stats-finished", review_stats)
         self.save_review_stats_cache_file()
 
@@ -190,10 +186,8 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         return True
 
     # writing new reviews spawns external helper
-    # FIXME: instead of the callback we should add proper gobject signals
     def spawn_write_new_review_ui(self, translated_app, version, iconname,
-                                  origin, parent_xid, datadir, callback,
-                                  done_callback=None):
+                                  origin, parent_xid, datadir):
         """ this spawns the UI for writing a new review and
             adds it automatically to the reviews DB """
         app = translated_app.get_untranslated_app(self.db)
@@ -210,13 +204,20 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
             cmd += ["--appname", utf8(app.appname)]
         spawn_helper = SpawnHelper(format="json")
         spawn_helper.connect(
-            "data-available", self._on_submit_review_data, app, callback)
-        if done_callback:
-            spawn_helper.connect("exited", done_callback)
-            spawn_helper.connect("error", done_callback)
+            "data-available", self._on_submit_review_data, app)
+        spawn_helper.connect("exited", self._on_exited_callback, app)
+        spawn_helper.connect("error", self._on_error_callback, app)
         spawn_helper.run(cmd)
 
-    def _on_submit_review_data(self, spawn_helper, review_json, app, callback):
+    def _on_exited_callback(self, spawn_helper, return_code, app):
+        # FIXME: send a proper error here instead!
+        self.emit("get-reviews-finished", app, [])
+
+    def _on_error_callback(self, spawn_helper, error_str, app):
+        # FIXME: send a proper error here instead!
+        self.emit("get-reviews-finished", app, [])
+
+    def _on_submit_review_data(self, spawn_helper, review_json, app):
         """ called when submit_review finished, when the review was send
             successfully the callback is triggered with the new reviews
         """
@@ -229,12 +230,12 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         if not app in self._reviews:
             self._reviews[app] = []
         self._reviews[app].insert(0, Review.from_piston_mini_client(review))
-        callback(app, self._reviews[app])
+        self.emit("get-reviews-finished", app, self._reviews[app])
 
-    def spawn_report_abuse_ui(self, review_id, parent_xid, datadir, callback):
+    def spawn_report_abuse_ui(self, review_id, parent_xid, datadir):
         """ this spawns the UI for reporting a review as inappropriate
             and adds the review-id to the internal hide list. once the
-            operation is complete it will call callback with the updated
+            operation is complete it will emit remove-review with the updated
             review list
         """
         cmd = [os.path.join(datadir, RNRApps.REPORT_REVIEW),
@@ -245,11 +246,10 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         spawn_helper = SpawnHelper("json")
         spawn_helper.connect("exited",
                              self._on_report_abuse_finished,
-                             review_id, callback)
+                             review_id)
         spawn_helper.run(cmd)
 
-    def _on_report_abuse_finished(self, spawn_helper, exitcode, review_id,
-        callback):
+    def _on_report_abuse_finished(self, spawn_helper, exitcode, review_id):
         """ called when report_absuse finished """
         LOG.debug("hide id %s " % review_id)
         if exitcode == 0:
@@ -258,12 +258,11 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
                     if str(review.id) == str(review_id):
                         # remove the one we don't want to see anymore
                         self._reviews[app].remove(review)
-                        callback(app, self._reviews[app], None, 'remove',
-                            review)
+                        self.emit("remove-review", app, review)
                         break
 
     def spawn_submit_usefulness_ui(self, review_id, is_useful, parent_xid,
-        datadir, callback):
+                                   datadir):
         cmd = [os.path.join(datadir, RNRApps.SUBMIT_USEFULNESS),
                "--review-id", "%s" % review_id,
                "--is-useful", "%s" % int(is_useful),
@@ -273,21 +272,20 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         spawn_helper = SpawnHelper(format="none")
         spawn_helper.connect("exited",
                              self._on_submit_usefulness_finished,
-                             review_id, is_useful, callback)
+                             review_id, is_useful)
         spawn_helper.connect("error",
                              self._on_submit_usefulness_error,
-                             review_id, callback)
+                             review_id)
         spawn_helper.run(cmd)
 
     def _on_submit_usefulness_finished(self, spawn_helper, res, review_id,
-        is_useful, callback):
+                                       is_useful):
         """ called when report_usefulness finished """
         # "Created", "Updated", "Not modified" -
         # once lp:~mvo/rnr-server/submit-usefulness-result-strings makes it
         response = spawn_helper._stdout
         if response == '"Not modified"':
-            self._on_submit_usefulness_error(spawn_helper, response, review_id,
-                callback)
+            self._on_submit_usefulness_error(spawn_helper, response, review_id)
             return
 
         LOG.debug("usefulness id %s " % review_id)
@@ -301,24 +299,23 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
                     review.usefulness_total = getattr(review,
                         "usefulness_total", 0) + 1
                     if is_useful:
-                        review.usefulness_favorable = getattr(review,
-                            "usefulness_favorable", 0) + 1
-                        callback(app, self._reviews[app], useful_votes,
-                            'replace', review)
+                        review.usefulness_favorable = getattr(
+                            review, "usefulness_favorable", 0) + 1
+                        self.emit("update-usefulness-votes", useful_votes)
+                        self.emit("replace-review", app, review)
                         break
 
-    def _on_submit_usefulness_error(self, spawn_helper, error_str, review_id,
-        callback):
-        LOG.warn("submit usefulness id=%s failed with error: %s" %
-                 (review_id, error_str))
-        for (app, reviews) in self._reviews.items():
-            for review in reviews:
-                if str(review.id) == str(review_id):
-                    review.usefulness_submit_error = True
-                    callback(app, self._reviews[app], None, 'replace', review)
-                    break
+    def _on_submit_usefulness_error(self, spawn_helper, error_str, review_id):
+            LOG.warn("submit usefulness id=%s failed with error: %s" %
+                     (review_id, error_str))
+            for (app, reviews) in self._reviews.items():
+                for review in reviews:
+                    if str(review.id) == str(review_id):
+                        review.usefulness_submit_error = True
+                        self.emit("replace-review", app, review)
+                        break
 
-    def spawn_delete_review_ui(self, review_id, parent_xid, datadir, callback):
+    def spawn_delete_review_ui(self, review_id, parent_xid, datadir):
         cmd = [os.path.join(datadir, RNRApps.DELETE_REVIEW),
                "--review-id", "%s" % review_id,
                "--parent-xid", "%s" % parent_xid,
@@ -327,13 +324,12 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         spawn_helper = SpawnHelper(format="none")
         spawn_helper.connect("exited",
                              self._on_delete_review_finished,
-                             review_id, callback)
+                             review_id)
         spawn_helper.connect("error", self._on_delete_review_error,
-                             review_id, callback)
+                             review_id)
         spawn_helper.run(cmd)
 
-    def _on_delete_review_finished(self, spawn_helper, res, review_id,
-        callback):
+    def _on_delete_review_finished(self, spawn_helper, res, review_id):
         """ called when delete_review finished"""
         LOG.debug("delete id %s " % review_id)
         for (app, reviews) in self._reviews.items():
@@ -341,11 +337,10 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
                 if str(review.id) == str(review_id):
                     # remove the one we don't want to see anymore
                     self._reviews[app].remove(review)
-                    callback(app, self._reviews[app], None, 'remove', review)
+                    self.emit("remove-review", app, review)
                     break
 
-    def _on_delete_review_error(self, spawn_helper, error_str, review_id,
-        callback):
+    def _on_delete_review_error(self, spawn_helper, error_str, review_id):
         """called if delete review errors"""
         LOG.warn("delete review id=%s failed with error: %s" % (review_id,
             error_str))
@@ -353,12 +348,10 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
             for review in reviews:
                 if str(review.id) == str(review_id):
                     review.delete_error = True
-                    callback(app, self._reviews[app], action='replace',
-                             single_review=review)
+                    self.emit("remove-review", app, review)
                     break
 
-    def spawn_modify_review_ui(self, parent_xid, iconname, datadir, review_id,
-        callback):
+    def spawn_modify_review_ui(self, parent_xid, iconname, datadir, review_id):
         """ this spawns the UI for writing a new review and
             adds it automatically to the reviews DB """
         cmd = [os.path.join(datadir, RNRApps.MODIFY_REVIEW),
@@ -370,13 +363,12 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         spawn_helper = SpawnHelper(format="json")
         spawn_helper.connect("data-available",
                              self._on_modify_review_finished,
-                             review_id, callback)
+                             review_id)
         spawn_helper.connect("error", self._on_modify_review_error,
-                             review_id, callback)
+                             review_id)
         spawn_helper.run(cmd)
 
-    def _on_modify_review_finished(self, spawn_helper, review_json, review_id,
-        callback):
+    def _on_modify_review_finished(self, spawn_helper, review_json, review_id):
         """called when modify_review finished"""
         LOG.debug("_on_modify_review_finished")
         #review_json = spawn_helper._stdout
@@ -388,12 +380,10 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
                     self._reviews[app].remove(review)
                     new_review = Review.from_piston_mini_client(mod_review)
                     self._reviews[app].insert(0, new_review)
-                    callback(app, self._reviews[app], action='replace',
-                             single_review=new_review)
+                    self.emit("replace-review", app, new_review)
                     break
 
-    def _on_modify_review_error(self, spawn_helper, error_str, review_id,
-        callback):
+    def _on_modify_review_error(self, spawn_helper, error_str, review_id):
         """called if modify review errors"""
         LOG.debug("modify review id=%s failed with error: %s" %
             (review_id, error_str))
@@ -401,6 +391,5 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
             for review in reviews:
                 if str(review.id) == str(review_id):
                     review.modify_error = True
-                    callback(app, self._reviews[app], action='replace',
-                             single_review=review)
+                    self.emit("replace-review", app, review)
                     break
