@@ -61,7 +61,7 @@ SOFTWARE_CENTER_TOS_LINK_NO_HEADER = "https://apps.ubuntu.com/cat/tos/plain/ "
 
 # version of the database, every time something gets added (like
 # terms for mime-type) increase this (but keep as a string!)
-DB_SCHEMA_VERSION = "6"
+DB_SCHEMA_VERSION = "7"
 
 # the default limit for a search
 DEFAULT_SEARCH_LIMIT = 10000
@@ -165,10 +165,54 @@ class XapianValues:
     SUPPORT_SITE_URL = 197
     VERSION_INFO = 198
     SC_SUPPORTED_DISTROS = 199
+    WEBSITE = 200
+    CURRENCY = 201
+    # this is used to provide a cataloged time if there is no a-x-i in use
+    # or if a-x-i is not available yet
+    DB_CATALOGED_TIME = 202
+
+
+class AppInfoFields:
+    ARCH = 'ARCH'
+    CHANNEL = 'CHANNEL'
+    DEB_LINE = 'DEB_LINE'
+    DEB_LINE_ORIG = 'DEB_LINE_ORIG'
+    CATEGORIES = 'CATEGORIES'
+    DATE_PUBLISHED = 'DATE_PUBLISHED'
+    DESCRIPTION = 'DESCRIPTION'
+    GENERIC_NAME = 'GENERIC_NAME'
+    GETTEXT_DOMAIN = 'GETTEXT_DOMAIN'
+    ICON = 'ICON'
+    ICON_URL = 'ICON_URL'
+    IGNORE = 'IGNORE'
+    KEYWORDS = 'KEYWORDS'
+    LICENSE = 'LICENSE'
+    LICENSE_KEY = 'LICENSE_KEY'
+    LICENSE_KEY_PATH = 'LICENSE_KEY_PATH'
+    MIMETYPE = 'MIMETYPE'
+    NAME = 'NAME'
+    NAME_UNTRANSLATED = 'NAME_UNTRANSLATED'
+    PACKAGE = 'PACKAGE'
+    POPCON = 'POPCON'
+    PPA = 'PPA'
+    PRICE = 'PRICE'
+    PURCHASED_DATE = 'PURCHASED_DATE'
+    SECTION = 'SECTION'
+    SIGNING_KEY_ID = 'SIGNING_KEY_ID'
+    SCREENSHOT_URLS = 'SCREENSHOT_URLS'
+    SUMMARY = 'SUMMARY'
+    SUPPORT_URL = 'SUPPORT_URL'
+    SUPPORTED_DISTROS = 'SUPPORTED_DISTROS'
+    TAGS = 'TAGS'
+    THUMBNAIL_URL = 'THUMBNAIL_URL'
+    TYPE = 'TYPE'
+    VERSION = 'VERSION'
+    VIDEO_URL = 'VIDEO_URL'
+    WEBSITE = 'WEBSITE'
+    CURRENCY = 'CURRENCY'
 
 
 # fake channels
-PURCHASED_NEEDS_REINSTALL_MAGIC_CHANNEL_NAME = "for-pay-needs-reinstall"
 AVAILABLE_FOR_PURCHASE_MAGIC_CHANNEL_NAME = "available-for-pay"
 
 
@@ -185,39 +229,37 @@ class CustomKeys:
 
 # pkg action state constants
 class PkgStates:
-    (
     # current
-    INSTALLED,
-    UNINSTALLED,
-    UPGRADABLE,
-    REINSTALLABLE,
+    INSTALLED = "installed"
+    UNINSTALLED = "uninstalled"
+    UPGRADABLE = "upgradable"
+    REINSTALLABLE = "reinstallable"
     # progress
-    INSTALLING,
-    REMOVING,
-    UPGRADING,
-    ENABLING_SOURCE,
-    INSTALLING_PURCHASED,
+    INSTALLING = "installing"
+    REMOVING = "removing"
+    UPGRADING = "upgrading"
+    ENABLING_SOURCE = "enabling_source"
+    INSTALLING_PURCHASED = "installing_purchase"
     # special
-    NEEDS_SOURCE,
-    NEEDS_PURCHASE,
-    PURCHASED_BUT_REPO_MUST_BE_ENABLED,
-    ERROR,
-    FORCE_VERSION,
+    NEEDS_SOURCE = "needs_source"
+    NEEDS_PURCHASE = "needs_purchase"
+    PURCHASED_BUT_REPO_MUST_BE_ENABLED = "purchased_but_repo_must_be_enabled"
+    ERROR = "error"
+    FORCE_VERSION = "force_version"
     # the package is not found in the DB or cache
-    NOT_FOUND,
+    NOT_FOUND = "not_found"
     # its purchased but not found for the current series
-    PURCHASED_BUT_NOT_AVAILABLE_FOR_SERIES,
-    # this *needs* to be last (for test_appdetails.py) and means
+    PURCHASED_BUT_NOT_AVAILABLE_FOR_SERIES = \
+                                    "purchased_but_not_available_for_series"
     # something went wrong and we don't have a state for this PKG
-    UNKNOWN,
-    ) = range(17)
+    UNKNOWN = "unknown"
 
 
 # visibility of non applications in the search results
 class NonAppVisibility:
-    (ALWAYS_VISIBLE,
-     MAYBE_VISIBLE,
-     NEVER_VISIBLE) = range(3)
+    ALWAYS_VISIBLE = "non-apps-always-visible"
+    MAYBE_VISIBLE = "non-apps-maybe-visible"
+    NEVER_VISIBLE = "non-apps-never-visible"
 
 
 # application actions
@@ -244,6 +286,11 @@ class SearchSeparators:
     PACKAGE = ","
 
 
+# action values for recommendations feedback
+class RecommenderFeedbackActions:
+    VIEWED = "viewed"
+    INSTALLED = "installed"
+
 # mouse event codes for back/forward buttons
 # TODO: consider whether we ought to get these values from gconf so that we
 #       can be sure to use the corresponding values used by Nautilus:
@@ -257,6 +304,9 @@ APP_INSTALL_PATH_DELIMITER = "__"
 
 #carousel app limit to override limit in .menu file for category
 TOP_RATED_CAROUSEL_LIMIT = 12
+WHATS_NEW_CAROUSEL_LIMIT = 9
+LOBBY_RECOMMENDATIONS_CAROUSEL_LIMIT = 9
+DETAILS_RECOMMENDATIONS_CAROUSEL_LIMIT = 4
 
 # Transaction ID for the "fake" purchase transaction
 PURCHASE_TRANSACTION_ID = "FakePurchaseTransactionID"
