@@ -25,20 +25,21 @@ import atexit
 import collections
 import dbus
 import dbus.service
-from dbus.mainloop.glib import DBusGMainLoop
-DBusGMainLoop(set_as_default=True)
-
 import gettext
+import glob
 import logging
 import os
 import re
 import subprocess
 import sys
-import xapian
-import glob
-
+import time
 import webbrowser
+import xapian
 
+from dbus.mainloop.glib import DBusGMainLoop
+DBusGMainLoop(set_as_default=True)
+
+from os.path import normpath
 from gettext import gettext as _
 
 # purely to initialize the netstatus
@@ -46,9 +47,15 @@ import softwarecenter.netstatus
 # make pyflakes shut up
 softwarecenter.netstatus.NETWORK_STATE
 
+from softwarecenter.backend.ubuntusso import UbuntuSSO
+
 # db imports
 from softwarecenter.db.application import Application
-from softwarecenter.db import DebFileApplication, DebFileOpenError
+from softwarecenter.db import (
+    DebFileApplication,
+    DebFileOpenError,
+    )
+from softwarecenter.db.utils import run_software_center_agent
 from softwarecenter.i18n import init_locale
 
 # misc imports
@@ -79,7 +86,10 @@ from softwarecenter.ui.gtk3.utils import (
     init_sc_css_provider,
 )
 from softwarecenter.version import VERSION
-from softwarecenter.db.database import StoreDatabase
+from softwarecenter.db.database import (
+    StoreDatabase,
+    get_reinstall_previous_purchases_query,
+    )
 try:
     from aptd_gtk3 import InstallBackendUI
     InstallBackendUI  # pyflakes
@@ -91,8 +101,10 @@ import softwarecenter.ui.gtk3.dialogs.deauthorize_dialog as deauthorize_dialog
 import softwarecenter.ui.gtk3.dialogs as dialogs
 
 from softwarecenter.ui.gtk3.SimpleGtkbuilderApp import SimpleGtkbuilderApp
-from softwarecenter.ui.gtk3.panes.installedpane import InstalledPane
-from softwarecenter.ui.gtk3.panes.availablepane import AvailablePane
+with ExecutionTime("import InstalledPane"):
+    from softwarecenter.ui.gtk3.panes.installedpane import InstalledPane
+with ExecutionTime("import AvailablePane"):
+    from softwarecenter.ui.gtk3.panes.availablepane import AvailablePane
 from softwarecenter.ui.gtk3.panes.historypane import HistoryPane
 from softwarecenter.ui.gtk3.panes.globalpane import GlobalPane
 from softwarecenter.ui.gtk3.panes.pendingpane import PendingPane
@@ -108,8 +120,8 @@ from softwarecenter.ui.gtk3.widgets.recommendations import (
     RecommendationsOptInDialog)
 
 from softwarecenter.config import get_config
-from softwarecenter.backend import get_install_backend
-from softwarecenter.backend.login_sso import get_sso_backend
+from softwarecenter.backend.installbackend import get_install_backend
+from softwarecenter.backend.login import get_login_backend
 from softwarecenter.backend.recagent import RecommenderAgent
 
 from softwarecenter.backend.channel import AllInstalledChannel
@@ -126,6 +138,7 @@ from gi.repository import Gdk
 
 LOG = logging.getLogger(__name__)
 PACKAGE_PREFIX = 'apt:'
+FILE_PREFIX = 'file:'
 # "apt:///" is a valid prefix for 'apt:pkgname' in alt+F2 in gnome
 PACKAGE_PREFIX_REGEX = re.compile('^%s(?:/{2,3})*' % PACKAGE_PREFIX)
 SEARCH_PREFIX = 'search:'
@@ -163,6 +176,11 @@ def parse_packages_args(packages):
             # remove the initial search prefix
             items[0] = items[0].replace(SEARCH_PREFIX, '', 1)
             search_text = SearchSeparators.REGULAR.join(items)
+        elif items[0].startswith(FILE_PREFIX):
+            # strip away the initial file: prefix, if present
+            items[0] = items[0].replace(FILE_PREFIX, '', 1)
+            # normalize the path to strip duplicate path separators, etc
+            items[0] = normpath(items[0])
         else:
             # strip away the initial apt: prefix, if present
             items[0] = re.sub(PACKAGE_PREFIX_REGEX, '', items[0])
@@ -210,7 +228,7 @@ class SoftwarecenterDbusController(dbus.service.Object):
         """ stop the dbus controller and remove from the bus """
         self.remove_from_connection()
 
-    @dbus.service.method('com.ubuntu.SoftwarecenterIFace')
+    @dbus.service.method('com.ubuntu.SoftwarecenterIFace', utf8_strings=True)
     def bringToFront(self, args):
         if args != 'nothing-to-show':
             self.parent.show_available_packages(args)
@@ -225,6 +243,10 @@ class SoftwarecenterDbusController(dbus.service.Object):
     def triggerCacheReload(self):
         self.parent.cache.emit("cache-ready")
 
+    @dbus.service.method('com.ubuntu.SoftwarecenterIFace')
+    def writeMemoryDump(self):
+        self.parent.write_memory_dump()
+
 
 class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
 
@@ -235,16 +257,16 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
 
     START_DBUS = True
 
-    def __init__(self, datadir, xapian_base_path, options, args=None):
+    def __init__(self, options, args=None):
         self.dbusControler = None
         if self.START_DBUS:
             # setup dbus and exit if there is another instance already running
             self.setup_dbus_or_bring_other_instance_to_front(args)
 
-        self.datadir = datadir
+        self.datadir = softwarecenter.paths.datadir
         super(SoftwareCenterAppGtk3, self).__init__(
-                                     datadir + "/ui/gtk3/SoftwareCenter.ui",
-                                     "software-center")
+            os.path.join(self.datadir, "ui", "gtk3", "SoftwareCenter.ui"),
+            "software-center")
         gettext.bindtextdomain("software-center", "/usr/share/locale")
         gettext.textdomain("software-center")
 
@@ -269,6 +291,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             # cache is opened later in run()
             self.cache.connect("cache-broken", self._on_apt_cache_broken)
 
+        xapian_base_path = softwarecenter.paths.XAPIAN_BASE_PATH
         with ExecutionTime("opening the xapiandb"):
             pathname = os.path.join(xapian_base_path, "xapian")
             self._use_axi = not options.disable_apt_xapian_index
@@ -292,13 +315,13 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
                 dialogs.error(None,
                               _("Sorry, can not open the software database"),
                               _("Please re-install the 'software-center' "
-                                "package."))
+                               "package."))
                 # FIXME: force rebuild by providing a dbus service for this
                 sys.exit(1)
 
         # additional icons come from app-install-data
         with ExecutionTime("building the icon cache"):
-            self.icons = get_sc_icon_theme(self.datadir)
+            self.icons = get_sc_icon_theme()
 
         # backend
         with ExecutionTime("creating the backend"):
@@ -319,7 +342,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         # for use when viewing previous purchases
         self.scagent = None
         self.sso = None
-        self.available_for_me_query = None
+        self.available_for_me_query = get_reinstall_previous_purchases_query()
 
         Gtk.Window.set_default_icon_name("softwarecenter")
 
@@ -330,24 +353,23 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         # initial css loading
         init_sc_css_provider(self.window_main,
                              settings,
-                             Gdk.Screen.get_default(),
-                             datadir)
+                             Gdk.Screen.get_default())
 
         # wire up the css provider to reconfigure on theme-changes
         self.window_main.connect("style-updated",
                                  self._on_style_updated,
                                  init_sc_css_provider,
                                  settings,
-                                 Gdk.Screen.get_default(),
-                                 datadir)
+                                 Gdk.Screen.get_default())
+        # workaround broken engines (LP: #1021308)
+        self.window_main.emit("style-updated")
 
         # register view manager and create view panes/widgets
-        with ExecutionTime("ViewManager"):
-            self.view_manager = ViewManager(self.notebook_view, options)
+        self.view_manager = ViewManager(self.notebook_view, options)
 
         with ExecutionTime("building panes"):
-            self.global_pane = GlobalPane(self.view_manager, self.datadir,
-                self.db, self.cache, self.icons)
+            self.global_pane = GlobalPane(self.view_manager, self.db,
+                                          self.cache, self.icons)
             self.vbox1.pack_start(self.global_pane, False, False, 0)
             self.vbox1.reorder_child(self.global_pane, 1)
 
@@ -360,7 +382,6 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
                                                 self.db,
                                                 self.distro,
                                                 self.icons,
-                                                self.datadir,
                                                 self.navhistory_back_action,
                                                 self.navhistory_forward_action)
             self.available_pane.connect("available-pane-created",
@@ -372,8 +393,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             self.installed_pane = InstalledPane(self.cache,
                                                 self.db,
                                                 self.distro,
-                                                self.icons,
-                                                self.datadir)
+                                                self.icons)
             self.installed_pane.connect("installed-pane-created",
                 self.on_installed_pane_created)
             self.view_manager.register(self.installed_pane,
@@ -383,8 +403,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             self.history_pane = HistoryPane(self.cache,
                                             self.db,
                                             self.distro,
-                                            self.icons,
-                                            self.datadir)
+                                            self.icons)
             self.view_manager.register(self.history_pane, ViewPages.HISTORY)
 
             # pending pane
@@ -402,18 +421,16 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         # reviews
         with ExecutionTime("create review loader"):
             self.review_loader = get_review_loader(self.cache, self.db)
-            # FIXME: add some kind of throttle, I-M-S here
-            self.review_loader.refresh_review_stats(
-                self.on_review_stats_loaded)
+            self.review_loader.connect(
+                "refresh-review-stats-finished", self.on_review_stats_loaded)
+            self.review_loader.refresh_review_stats()
             #load usefulness votes from server when app starts
             self.useful_cache = UsefulnessCache(True)
             self.setup_database_rebuilding_listener()
 
-        with ExecutionTime("create plugin manager"):
-            # open plugin manager and load plugins
-            self.plugin_manager = PluginManager(self,
-                SOFTWARE_CENTER_PLUGIN_DIRS)
-            self.plugin_manager.load_plugins()
+        # open plugin manager and load plugins
+        self.plugin_manager = PluginManager(self, SOFTWARE_CENTER_PLUGIN_DIRS)
+        self.plugin_manager.load_plugins()
 
         # setup window name and about information (needs branding)
         name = self.distro.get_app_name()
@@ -451,7 +468,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         # menu item if Unity is not currently running
         if is_unity_running():
             self.menuitem_add_to_launcher.set_active(
-                                self.available_pane.add_to_launcher_enabled)
+                self.config.add_to_unity_launcher)
         else:
             self.menu_view.remove(self.add_to_launcher_separator)
             self.menu_view.remove(self.menuitem_add_to_launcher)
@@ -459,8 +476,6 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         # run s-c-agent update
         if options.disable_buy or not self.distro.PURCHASE_APP_URL:
             self.menu_file.remove(self.menuitem_reinstall_purchases)
-            if not (options.enable_lp or och):
-                self.menu_file.remove(self.separator_login)
         else:
             # running the agent will trigger a db reload so we do it later
             GObject.timeout_add_seconds(3, self._run_software_center_agent)
@@ -478,7 +493,6 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
 
         # keep track of the current active pane
         self.active_pane = self.available_pane
-        self.window_main.connect("realize", self.on_realize)
 
         # launchpad integration help, its ok if that fails
         try:
@@ -491,13 +505,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
     # helper
     def _run_software_center_agent(self):
         """ helper that triggers the update-software-center-agent helper """
-        sc_agent_update = os.path.join(
-            self.datadir, "update-software-center-agent")
-        (pid, stdin, stdout, stderr) = GObject.spawn_async(
-            [sc_agent_update, "--datadir", self.datadir],
-            flags=GObject.SPAWN_DO_NOT_REAP_CHILD)
-        GObject.child_watch_add(
-            pid, self._on_update_software_center_agent_finished)
+        run_software_center_agent(self.db)
 
     def _run_expunge_cache_helper(self):
         """ helper that expires the piston-mini-client cache """
@@ -545,9 +553,6 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             pass
 
     # callbacks
-    def on_realize(self, widget):
-        pass
-
     def on_available_pane_created(self, widget):
         self.available_pane.searchentry.grab_focus()
         self._update_recommendations_menuitem(
@@ -593,13 +598,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             self._recommender_agent = RecommenderAgent()
         return self._recommender_agent
 
-    def _on_update_software_center_agent_finished(self, pid, condition):
-        LOG.info("software-center-agent finished with status %i" %
-            os.WEXITSTATUS(condition))
-        if os.WEXITSTATUS(condition) == 0:
-            self.db.reopen()
-
-    def on_review_stats_loaded(self, reviews):
+    def on_review_stats_loaded(self, loader, reviews):
         LOG.debug("on_review_stats_loaded: '%s'" % len(reviews))
 
     def destroy(self):
@@ -624,9 +623,6 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         if Gtk.main_level() == 0:
             LOG.info("closing before the regular main loop was run")
             sys.exit(0)
-        # this is the case when it regularly runs
-        if hasattr(self, "glaunchpad"):
-            self.glaunchpad.shutdown()
         self.save_state()
         self.destroy()
 
@@ -736,32 +732,9 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             if self.menuitem_go_forward.get_sensitive():
                 self.menuitem_go_forward.activate()
 
-    def _on_lp_login(self, lp, token):
-        self._lp_login_successful = True
-        private_archives = self.glaunchpad.get_subscribed_archives()
-        channel_manager = self.view_switcher.get_model().channel_manager
-        channel_manager.feed_in_private_sources_list_entries(
-            private_archives)
-
-    def _on_sso_login(self, sso, oauth_result):
-        self._sso_login_successful = True
-        # appmanager needs to know about the oauth token for the reinstall
-        # previous purchases add_license_key call
-        self.app_manager.oauth_token = oauth_result
-        self.scagent.query_available_for_me()
-
     def _on_style_updated(self, widget, init_css_callback, *args):
         init_css_callback(widget, *args)
 
-    def _available_for_me_result(self, scagent, result_list):
-        #print "available_for_me_result", result_list
-        from softwarecenter.db.update import (
-            add_from_purchased_but_needs_reinstall_data)
-        available = add_from_purchased_but_needs_reinstall_data(result_list,
-            self.db, self.cache)
-        self.available_for_me_query = available
-        self.available_pane.on_previous_purchases_activated(available)
-
     def get_icon_filename(self, iconname, iconsize):
         iconinfo = self.icons.lookup_icon(iconname, iconsize, 0)
         if not iconinfo:
@@ -834,38 +807,53 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         # return False to ensure that a possible GObject.timeout_add ends
         return False
 
-    def on_menuitem_launchpad_private_ppas_activate(self, menuitem):
-        from backend.launchpad import GLaunchpad
-        self.glaunchpad = GLaunchpad()
-        self.glaunchpad.connect("login-successful", self._on_lp_login)
-        from view.logindialog import LoginDialog
-        d = LoginDialog(self.glaunchpad, self.datadir, parent=self.window_main)
-        d.login()
+    def _on_reinstall_purchased_login(self, sso, oauth_result):
+        self._sso_login_successful = True
+        # appmanager needs to know about the oauth token for the reinstall
+        # previous purchases add_license_key call
+        self.app_manager.oauth_token = oauth_result
 
-    def _create_dbus_sso(self):
-        # see bug #773214 for the rationale, do not translate the appname
-        #appname = _("Ubuntu Software Center")
-        appname = SOFTWARE_CENTER_NAME_KEYRING
-        help_text = _("To reinstall previous purchases, sign in to the "
-            "Ubuntu Single Sign-On account you used to pay for them.")
-        #window = self.window_main.get_window()
-        #xid = self.get_window().xid
-        xid = 0
-        self.sso = get_sso_backend(xid,
-                                   appname,
-                                   help_text)
-        self.sso.connect("login-successful", self._on_sso_login)
+        # the software-center-agent will ensure the previous-purchases
+        # get merged in
+        self._run_software_center_agent()
 
-    def _login_via_dbus_sso(self):
-        self._create_dbus_sso()
-        self.sso.login()
+        # show spinner as this may take some time, the spinner will
+        # automatically go away when a DB refreshes
+        self.available_pane.show_appview_spinner()
 
-    def _create_scagent_if_needed(self):
-        if not self.scagent:
-            from softwarecenter.backend.scagent import SoftwareCenterAgent
-            self.scagent = SoftwareCenterAgent()
-            self.scagent.connect("available-for-me",
-                                 self._available_for_me_result)
+        # show previous purchased
+        self.available_pane.on_previous_purchases_activated(
+            self.available_for_me_query)
+
+    def on_menuitem_reinstall_purchases_activate(self, menuitem):
+        self.view_manager.set_active_view(ViewPages.AVAILABLE)
+        self.view_manager.search_entry.clear_with_no_signal()
+        # its ok to use the sync version here to get the token, this is
+        # very quick
+        helper = UbuntuSSO()
+        token = helper.find_oauth_token_sync()
+        if token:
+            # trigger a software-center-agent run to ensure the merged in
+            # purchases are fresh
+            self._run_software_center_agent()
+            # we already have the list of available items, so just show it
+            # (no need for spinner here)
+            self.available_pane.on_previous_purchases_activated(
+                    self.available_for_me_query)
+        else:
+            # see bug #773214 for the rationale, do not translate the appname
+            #appname = _("Ubuntu Software Center")
+            appname = SOFTWARE_CENTER_NAME_KEYRING
+            help_text = _(
+                "To reinstall previous purchases, sign in to the "
+                "Ubuntu Single Sign-On account you used to pay for them.")
+            #window = self.window_main.get_window()
+            #xid = self.get_window().xid
+            xid = 0
+            self.sso = get_login_backend(xid, appname, help_text)
+            self.sso.connect(
+                "login-successful", self._on_reinstall_purchased_login)
+            self.sso.login()
 
     def on_menuitem_recommendations_activate(self, menu_item):
         rec_panel = self.available_pane.cat_view.recommended_for_you_panel
@@ -879,21 +867,6 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             if res == Gtk.ResponseType.YES:
                 rec_panel.opt_in_to_recommendations_service()
 
-    def on_menuitem_reinstall_purchases_activate(self, menuitem):
-        self.view_manager.set_active_view(ViewPages.AVAILABLE)
-        self.view_manager.search_entry.clear_with_no_signal()
-        if self.available_for_me_query:
-            # we already have the list of available items, so just show it
-            # (no need for spinner here)
-            self.available_pane.on_previous_purchases_activated(
-                    self.available_for_me_query)
-        else:
-            # show spinner as this may take some time
-            self.available_pane.show_appview_spinner()
-            # fetch the list of available items and show it
-            self._create_scagent_if_needed()
-            self._login_via_dbus_sso()
-
     def on_menuitem_deauthorize_computer_activate(self, menuitem):
 
         # FIXME: need Ubuntu SSO username here
@@ -1127,7 +1100,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         if self.menuitem_view.blocked:
             return
         from softwarecenter.db.appfilter import get_global_filter
-        if get_global_filter().supported_only == True:
+        if get_global_filter().supported_only is True:
             get_global_filter().supported_only = False
 
             self.available_pane.refresh_apps()
@@ -1140,7 +1113,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         if self.menuitem_view.blocked:
             return
         from softwarecenter.db.appfilter import get_global_filter
-        if get_global_filter().supported_only == False:
+        if get_global_filter().supported_only is False:
             get_global_filter().supported_only = True
 
             self.available_pane.refresh_apps()
@@ -1177,7 +1150,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         vm.nav_forward()
 
     def on_menuitem_add_to_launcher_toggled(self, menu_item):
-        self.available_pane.add_to_launcher_enabled = menu_item.get_active()
+        self.config.add_to_unity_launcher = menu_item.get_active()
 
 # Help Menu
     def on_menuitem_about_activate(self, widget):
@@ -1198,7 +1171,7 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
 
     def _ask_and_repair_broken_cache(self):
         # wait until the window window is available
-        if self.window_main.props.visible == False:
+        if self.window_main.props.visible is False:
             GObject.timeout_add_seconds(1, self._ask_and_repair_broken_cache)
             return
         if dialogs.confirm_repair_broken_cache(self.window_main,
@@ -1352,9 +1325,9 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             self.view_manager.set_active_view(ViewPages.AVAILABLE)
 
     def restore_state(self):
-        if self.config.has_option("general", "size"):
-            (x, y) = self.config.get("general", "size").split(",")
-            self.window_main.set_default_size(int(x), int(y))
+        (x, y) = self.config.app_window_size
+        if x > 0 and y > 0:
+            self.window_main.set_default_size(x, y)
         else:
             # on first launch, specify the default window size to take
             # advantage of the available screen real estate (but set a
@@ -1364,17 +1337,8 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
             self.window_main.set_default_size(
                                         min(int(.85 * screen_width), 1200),
                                         min(int(.85 * screen_height), 800))
-        if (self.config.has_option("general", "maximized") and
-            self.config.getboolean("general", "maximized")):
+        if self.config.app_window_maximized:
             self.window_main.maximize()
-        if self.config.has_option("general", "add_to_launcher"):
-            self.available_pane.add_to_launcher_enabled = (
-                    self.config.getboolean(
-                    "general",
-                    "add_to_launcher"))
-        else:
-            # initial default state is to add to launcher, per spec
-            self.available_pane.add_to_launcher_enabled = True
 
     def save_state(self):
         LOG.debug("save_state")
@@ -1383,26 +1347,27 @@ class SoftwareCenterAppGtk3(SimpleGtkbuilderApp):
         if window is None:
             return
         maximized = window.get_state() & Gdk.WindowState.MAXIMIZED
-        if maximized:
-            self.config.set("general", "maximized", "True")
-        else:
-            self.config.set("general", "maximized", "False")
+        self.config.app_window_maximized = maximized
+        if not maximized:
             # size only matters when non-maximized
             size = self.window_main.get_size()
-            self.config.set("general", "size", "%s, %s" % (size[0], size[1]))
-        if self.available_pane.add_to_launcher_enabled:
-            self.config.set("general", "add_to_launcher", "True")
-        else:
-            self.config.set("general", "add_to_launcher", "False")
-        # store the recommender values
-        self.config.set("general",
-                        "recommender_uuid",
-                        self._get_recommender_agent().recommender_uuid)
-        self.config.set("general",
-                        "recommender_profile_id",
-                        self._get_recommender_agent().recommender_profile_id)
+            self.config.app_window_size = [size[0], size[1]]
         self.config.write()
 
+    def write_memory_dump(self, fname=None):
+        # trigger this e.g. with:
+        #  dbus-send --print-reply --session --dest=com.ubuntu.Softwarecenter \
+        #     /com/ubuntu/Softwarecenter  \
+        #     com.ubuntu.SoftwarecenterIFace.writeMemoryDump
+        if fname is None:
+            fname = "software-center_%s.meliae" % time.strftime(
+                "%Y%m%d_%H%M%S")
+        try:
+            from meliae import scanner
+            scanner.dump_all_objects(fname)
+        except Exception:
+            LOG.exception("write_memory_dump failed")
+
     def run(self, args):
         # show window as early as possible
         self.window_main.show_all()
