@@ -24,8 +24,7 @@ from gi.repository import Gtk, Gdk
 import logging
 import xapian
 
-from softwarecenter.backend import get_install_backend
-from softwarecenter.db.database import Application
+from softwarecenter.backend.installbackend import get_install_backend
 from softwarecenter.db.enquire import AppEnquire
 from softwarecenter.enums import (
     DEFAULT_SEARCH_LIMIT,
@@ -35,7 +34,6 @@ from softwarecenter.enums import (
 )
 from softwarecenter.utils import (
     ExecutionTime,
-    utf8,
     wait_for_apt_cache_ready,
 )
 
@@ -45,88 +43,13 @@ from softwarecenter.ui.gtk3.widgets.spinner import SpinnerNotebook
 from softwarecenter.ui.gtk3.widgets.searchaid import SearchAid
 
 from softwarecenter.ui.gtk3.views.appview import AppView
-from softwarecenter.ui.gtk3.views.appdetailsview import AppDetailsView
+
+from softwarecenter.ui.gtk3.session.displaystate import DisplayState
 
 from basepane import BasePane
 
 LOG = logging.getLogger(__name__)
 
-# for DisplayState attribute type-checking
-from softwarecenter.db.categories import Category
-from softwarecenter.backend.channel import SoftwareChannel
-from softwarecenter.db.appfilter import AppFilter
-
-
-class DisplayState(object):
-
-    _attrs = {'category': (type(None), Category),
-              'channel': (type(None), SoftwareChannel),
-              'subcategory': (type(None), Category),
-              'search_term': (str,),
-              'application': (type(None), Application),
-              'limit': (int,),
-              'filter': (type(None), AppFilter),
-              'vadjustment': (float, ),
-            }
-
-    def __init__(self):
-        self.category = None
-        self.channel = None
-        self.subcategory = None
-        self.search_term = ""
-        self.application = None
-        self.limit = 0
-        self.filter = None
-        self.vadjustment = 0.0
-
-    def __setattr__(self, name, val):
-        attrs = self._attrs
-        if name not in attrs:
-            raise AttributeError("The attr name \"%s\" is not permitted" %
-                name)
-            Gtk.main_quit()
-        if not isinstance(val, attrs[name]):
-            msg = "Attribute %s expects %s, got %s" % (name, attrs[name],
-                type(val))
-            raise TypeError(msg)
-            Gtk.main_quit()
-        return object.__setattr__(self, name, val)
-
-    def __str__(self):
-        s = utf8('%s %s "%s" %s %s') % \
-                                  (self.category,
-                                   self.subcategory,
-                                   self.search_term,
-                                   self.application,
-                                   self.channel)
-        return s
-
-    def copy(self):
-        state = DisplayState()
-        state.channel = self.channel
-        state.category = self.category
-        state.subcategory = self.subcategory
-        state.search_term = self.search_term
-        state.application = self.application
-        state.limit = self.limit
-        if self.filter:
-            state.filter = self.filter.copy()
-        else:
-            state.filter = None
-        state.vadjustment = self.vadjustment
-        return state
-
-    def reset(self):
-        self.channel = None
-        self.category = None
-        self.subcategory = None
-        self.search_term = ""
-        self.application = None
-        self.limit = 0
-        if self.filter:
-            self.filter.reset()
-        self.vadjustment = 0.0
-
 
 class SoftwarePane(Gtk.VBox, BasePane):
     """ Common base class for AvailablePane and InstalledPane"""
@@ -145,7 +68,7 @@ class SoftwarePane(Gtk.VBox, BasePane):
     }
     PADDING = 6
 
-    def __init__(self, cache, db, distro, icons, datadir, show_ratings=True):
+    def __init__(self, cache, db, distro, icons, show_ratings=True):
 
         Gtk.VBox.__init__(self)
         BasePane.__init__(self)
@@ -159,7 +82,6 @@ class SoftwarePane(Gtk.VBox, BasePane):
         self.db = db
         self.distro = distro
         self.icons = icons
-        self.datadir = datadir
         self.show_ratings = show_ratings
         self.backend = get_install_backend()
         self.nonapps_visible = NonAppVisibility.MAYBE_VISIBLE
@@ -211,8 +133,9 @@ class SoftwarePane(Gtk.VBox, BasePane):
         self.search_aid = SearchAid(self)
         self.box_app_list.pack_start(self.search_aid, False, False, 0)
 
-        self.app_view = AppView(self.db, self.cache,
-                                self.icons, self.show_ratings)
+        with ExecutionTime("SoftwarePane.AppView"):
+            self.app_view = AppView(self.db, self.cache,
+                                    self.icons, self.show_ratings)
         self.app_view.connect("sort-method-changed",
             self.on_app_view_sort_method_changed)
 
@@ -227,11 +150,15 @@ class SoftwarePane(Gtk.VBox, BasePane):
         self.scroll_details = Gtk.ScrolledWindow()
         self.scroll_details.set_policy(Gtk.PolicyType.AUTOMATIC,
                                         Gtk.PolicyType.AUTOMATIC)
-        self.app_details_view = AppDetailsView(self.db,
-                                               self.distro,
-                                               self.icons,
-                                               self.cache,
-                                               self.datadir)
+        # delayed import gives ~1s speedup until visible on the raspi
+        with ExecutionTime("import AppDetailsView"):
+            from softwarecenter.ui.gtk3.views.appdetailsview import (
+                    AppDetailsView)
+        with ExecutionTime("SoftwarePane.AppDetailsView"):
+            self.app_details_view = AppDetailsView(self.db,
+                                                   self.distro,
+                                                   self.icons,
+                                                   self.cache)
         self.app_details_view.connect(
             "different-application-selected", self.on_application_activated)
         self.scroll_details.add(self.app_details_view)
@@ -253,31 +180,43 @@ class SoftwarePane(Gtk.VBox, BasePane):
         Atk.Object.set_name(widget.get_accessible(), name)
 
     def on_cache_ready(self, cache):
-        " refresh the application list when the cache is re-opened "
+        """Refresh the application list when the cache is re-opened."""
         LOG.debug("on_cache_ready")
 
     @wait_for_apt_cache_ready
     def on_application_activated(self, appview, app):
-        """callback when an app is clicked"""
-        LOG.debug("on_application_activated: '%s'" % app)
+        """Callback when an app is clicked."""
+        LOG.debug("%r.on_application_activated: %r",
+                  self.__class__.__name__, app)
 
         self.state.application = app
 
         vm = get_viewmanager()
-        vm.display_page(self, SoftwarePane.Pages.DETAILS, self.state,
-            self.display_details_page)
+        # self.Pages will access the Page definition of each child correctly
+        vm.display_page(self, self.Pages.DETAILS, self.state)
+
+    def on_application_selected(self, widget, app):
+        """Stub implementation."""
+        LOG.debug("%r.on_application_selected: %r",
+                  self.__class__.__name__, app)
+
+    def enter_page(self, page, state):
+        if page == self.page.DETAILS:
+            self.display_details_page(state)
+
+    def is_applist_view_showing(self):
+        """Return True if we are in the applist view """
+        list_page = getattr(self.Pages, 'LIST', -1)
+        return self.notebook.get_current_page() == list_page
+
+    def is_app_details_view_showing(self):
+        """Return True if we are in the app_details view."""
+        details_page = getattr(self.Pages, 'DETAILS', -1)
+        return self.notebook.get_current_page() == details_page
 
     def show_app(self, app):
         self.on_application_activated(None, app)
 
-    def on_nav_back_clicked(self, widget):
-        vm = get_viewmanager()
-        vm.nav_back()
-
-    def on_nav_forward_clicked(self, widget):
-        vm = get_viewmanager()
-        vm.nav_forward()
-
     def on_query_complete(self, enquirer):
         self.emit("app-list-changed", len(enquirer.matches))
         self.app_view.display_matches(enquirer.matches,
@@ -444,7 +383,7 @@ class SoftwarePane(Gtk.VBox, BasePane):
                                 nonapps_visible=self.nonapps_visible,
                                 filter=self.state.filter)
 
-    def display_details_page(self, page, view_state):
+    def display_details_page(self, view_state):
         self.app_details_view.show_app(view_state.application)
         self.action_bar.unset_label()
         return True
@@ -488,36 +427,13 @@ class SoftwarePane(Gtk.VBox, BasePane):
 
     def on_search_terms_changed(self, terms):
         """Stub implementation."""
-        pass
 
     def on_db_reopen(self, db):
         """Stub implementation."""
         LOG.debug("%r: on_db_reopen (db is %r).", self.__class__.__name__, db)
 
-    def is_category_view_showing(self):
-        """Stub implementation."""
-        pass
-
-    def is_applist_view_showing(self):
-        """Stub implementation."""
-        pass
-
-    def is_app_details_view_showing(self):
-        """Stub implementation."""
-        pass
-
-    def get_current_app(self):
-        """Stub implementation."""
-        pass
-
-    def on_application_selected(self, widget, app):
-        """Stub implementation."""
-        pass
-
     def get_current_category(self):
         """Stub implementation."""
-        pass
 
     def unset_current_category(self):
         """Stub implementation."""
-        pass
