@@ -51,7 +51,7 @@ from softwarecenter.paths import SOFTWARE_CENTER_CONFIG_DIR
 from softwarecenter.enums import Icons, SOFTWARE_CENTER_NAME_KEYRING
 from softwarecenter.config import get_config
 from softwarecenter.distro import get_distro, get_current_arch
-from softwarecenter.backend.login_sso import get_sso_backend
+from softwarecenter.backend.login import get_login_backend
 from softwarecenter.backend.reviews import Review
 from softwarecenter.db.database import Application
 from softwarecenter.gwibber_helper import GwibberHelper, GwibberHelperMock
@@ -449,7 +449,7 @@ class BaseApp(SimpleGtkbuilderApp):
         login_window_xid = self._get_parent_xid_for_login_window()
         help_text = _("To review software or to report abuse you need to "
                       "sign in to a Ubuntu Single Sign-On account.")
-        self.sso = get_sso_backend(login_window_xid,
+        self.sso = get_login_backend(login_window_xid,
                                    SOFTWARE_CENTER_NAME_KEYRING, help_text)
         self.sso.connect("login-successful", self._maybe_login_successful)
         self.sso.connect("login-canceled", self._login_canceled)
@@ -716,6 +716,10 @@ class SubmitReviewsApp(BaseApp):
             self.quit(2)
 
     def _setup_details(self, widget, app, iconname, version, display_name):
+        if app is None:
+            logging.warning('Can not setup details since app is None.')
+            return
+
         # icon shazam
         try:
             icon = self.icons.load_icon(iconname, self.APP_ICON_SIZE, 0)
@@ -915,15 +919,8 @@ class SubmitReviewsApp(BaseApp):
             self._on_multiple_gwibber_accounts()
 
     def _get_gwibber_prefs(self):
-        if self.config.has_option("reviews", "gwibber_send"):
-            send = self.config.getboolean("reviews", "gwibber_send")
-        else:
-            send = False
-
-        if self.config.has_option("reviews", "account_id"):
-            account_id = self.config.get("reviews", "account_id")
-        else:
-            account_id = False
+        send = self.config.reviews_post_via_gwibber
+        account_id = self.config.reviews_gwibber_account_id
 
         return {
             "gwibber_send": send,
@@ -1128,13 +1125,9 @@ class SubmitReviewsApp(BaseApp):
             BaseApp.on_transmit_success(self, api, trans)
 
     def _save_gwibber_state(self, gwibber_send, account_id):
-        if not self.config.has_section("reviews"):
-            self.config.add_section("reviews")
-
-        self.config.set("reviews", "gwibber_send", str(gwibber_send))
+        self.config.reviews_post_via_gwibber = gwibber_send
         if account_id:
-            self.config.set("reviews", "account_id", account_id)
-
+            self.config.reviews_gwibber_account_id = account_id
         self.config.write()
 
     def _gwibber_message(self, max_len=140):
