@@ -42,6 +42,7 @@ from softwarecenter.enums import (AppActions,
                                   Icons,
                                   SOFTWARE_CENTER_PKGNAME)
 from softwarecenter.utils import (is_unity_running,
+                                  is_gnome_shell_running,
                                   upstream_version,
                                   get_exec_line_from_desktop,
                                   SimpleFileDownloader,
@@ -50,10 +51,6 @@ from softwarecenter.distro import get_distro
 from softwarecenter.backend.weblive import get_weblive_backend
 from softwarecenter.ui.gtk3.dialogs import error
 
-# FIXME: this is needed for the recommendations but really should become
-#        a widget or something generic instead
-from softwarecenter.ui.gtk3.views.catview_gtk import CategoriesViewGtk
-
 from softwarecenter.ui.gtk3.em import StockEms, em
 from softwarecenter.ui.gtk3.drawing import color_to_hex
 from softwarecenter.ui.gtk3.session.appmanager import get_appmanager
@@ -74,11 +71,13 @@ from softwarecenter.ui.gtk3.gmenusearch import GMenuSearcher
 
 import softwarecenter.ui.gtk3.dialogs as dialogs
 
+from softwarecenter.ui.gtk3.models.appstore2 import AppPropertiesHelper
+
 from softwarecenter.hw import get_hw_missing_long_description
 from softwarecenter.region import REGION_WARNING_STRING
 
 from softwarecenter.backend.reviews import get_review_loader
-from softwarecenter.backend import get_install_backend
+from softwarecenter.backend.installbackend import get_install_backend
 
 
 LOG = logging.getLogger(__name__)
@@ -102,6 +101,8 @@ class StatusBar(Gtk.Alignment):
         self._bg = [1, 1, 1, 0.3]
 
         self.connect("style-updated", self.on_style_updated)
+        # workaround broken engines (LP: #1021308)
+        self.emit("style-updated")
 
     def on_style_updated(self, widget):
         context = self.get_style_context()
@@ -364,14 +365,7 @@ class PackageStatusBar(StatusBar):
             elif state == PkgStates.INSTALLED:
                 self.set_button_label(_('Remove'))
         elif state == PkgStates.NEEDS_PURCHASE:
-            # FIXME:  need to determine the currency dynamically once we can
-            #         get that info from the software-center-agent/payments
-            #         service.
-            # NOTE:  the currency string for this label is purposely not
-            #        translatable when hardcoded, since it (currently)
-            #        won't vary based on locale and as such we don't want
-            #        it translated
-            self.set_label("US$ %s" % app_details.price)
+            self.set_label("%s" % (app_details.price))
             if (app_details.hardware_requirements_satisfied and
                 app_details.region_requirements_satisfied):
                 self.set_button_label(_(u'Buy\u2026'))
@@ -593,7 +587,7 @@ class Addon(Gtk.HBox):
 
         loader = self.get_ancestor(AppDetailsView).review_loader
         stats = loader.get_review_stats(self.app)
-        if stats != None:
+        if stats is not None:
             rating = Star()
             #~ rating.set_visible_window(False)
             rating.set_size_small()
@@ -671,7 +665,7 @@ class AddonsTable(Gtk.VBox):
 
             addon = Addon(self.db, self.icons, addon_name)
             #addon.pkgname.connect("clicked", not yet suitable for use)
-            addon.set_active(self.cache[addon_name].installed != None)
+            addon.set_active(self.cache[addon_name].installed is not None)
             addon.checkbutton.connect("toggled",
                                       self.addons_manager.mark_changes)
             self.pack_start(addon, False, False, 0)
@@ -768,8 +762,7 @@ class AddonsManager(object):
             if addon in self.addons_to_install:
                 self.addons_to_install.remove(addon)
         self.status_bar.configure()
-        GObject.idle_add(self.view.update_totalsize,
-                         priority=GObject.PRIORITY_LOW)
+        self.view.update_totalsize()
 
     def configure(self, pkgname, update_addons=True):
         self.addons_to_install = []
@@ -786,8 +779,7 @@ class AddonsManager(object):
         self.addons_to_install = []
         self.addons_to_remove = []
         self.configure(self.view.app.pkgname)
-        GObject.idle_add(self.view.update_totalsize,
-                         priority=GObject.PRIORITY_LOW)
+        self.view.update_totalsize()
 
 
 _asset_cache = {}
@@ -813,34 +805,43 @@ class AppDetailsView(Viewport):
                         (GObject.TYPE_PYOBJECT, )),
                     }
 
-    def __init__(self, db, distro, icons, cache, datadir):
+    def __init__(self, db, distro, icons, cache):
         Viewport.__init__(self)
         # basic stuff
         self.db = db
         self.distro = distro
         self.icons = icons
         self.cache = cache
+        # app specific data
+        self.app = None
+        self.app_details = None
+        self.pkg_state = None
+        self.cache.connect("query-total-size-on-install-done",
+                           self._on_query_total_size_on_install_done)
         self.backend = get_install_backend()
         self.cache.connect("cache-ready", self._on_cache_ready)
         self.connect("destroy", self._on_destroy)
-        self.datadir = datadir
-        self.app = None
-        self.appdetails = None
+        self.datadir = softwarecenter.paths.datadir
         self.addons_to_install = []
         self.addons_to_remove = []
+        self.properties_helper = AppPropertiesHelper(
+                self.db, self.cache, self.icons)
         # reviews
         self.review_loader = get_review_loader(self.cache, self.db)
+        self.review_loader.connect(
+            "get-reviews-finished", self._on_reviews_ready_callback)
+        self.review_loader.connect(
+            "remove-review", self._on_remove_review_callback)
+        self.review_loader.connect(
+            "replace-review", self._on_replace_review_callback)
+        self.review_loader.connect("update-usefulness-votes",
+                                   self._on_update_usefulness_votes_callback)
 
         # ui specific stuff
         self.set_shadow_type(Gtk.ShadowType.NONE)
         self.set_name("view")
 
         self.section = None
-        # app specific data
-        self.app = None
-        self.app_details = None
-        self.pkg_state = None
-
         self.reviews = UIReviewsList(self)
 
         self.adjustment_value = None
@@ -887,6 +888,9 @@ class AppDetailsView(Viewport):
 
     def _on_destroy(self, widget):
         self.cache.disconnect_by_func(self._on_cache_ready)
+        self.review_loader.disconnect_by_func(self._on_reviews_ready_callback)
+        self.review_loader.disconnect_by_func(self._on_remove_review_callback)
+        self.review_loader.disconnect_by_func(self._on_replace_review_callback)
 
     def _cache_art_assets(self):
         global _asset_cache
@@ -953,18 +957,12 @@ class AppDetailsView(Viewport):
     def _do_load_reviews(self):
         self.reviews.show_spinner_with_message(_('Checking for reviews...'))
         self.review_loader.get_reviews(
-            self.app, self._reviews_ready_callback,
+            self.app,
             page=self._reviews_server_page,
             language=self._reviews_server_language,
             sort=self._review_sort_method,
             relaxed=self._reviews_relaxed)
 
-    def _review_update_single(self, action, review):
-        if action == 'replace':
-            self.reviews.replace_review(review)
-        elif action == 'remove':
-            self.reviews.remove_review(review)
-
     def _update_review_stats_widget(self, stats):
         if stats:
             # ensure that the review UI knows about the stats
@@ -979,17 +977,28 @@ class AppDetailsView(Viewport):
     def _submit_reviews_done_callback(self, spawner, error):
         self.reviews.new_review.enable()
 
-    def _reviews_ready_callback(self, app, reviews_data, my_votes=None,
-                                action=None, single_review=None):
+    def _on_remove_review_callback(self, loader, app, review):
+        self.reviews.remove_review(review)
+        self.reviews.configure_reviews_ui()
+
+    def _on_replace_review_callback(self, loader, app, review):
+        self.reviews.replace_review(review)
+        self.reviews.configure_reviews_ui()
+
+    def _on_update_usefulness_votes_callback(self, loader, my_votes):
+        self.reviews.update_useful_votes(my_votes)
+        self.reviews.configure_reviews_ui()
+
+    def _on_reviews_ready_callback(self, loader, app, reviews_data):
         """ callback when new reviews are ready, cleans out the
             old ones
         """
-        LOG.debug("_review_ready_callback: %s" % app)
+        LOG.debug("_review_ready_callback: %s " % app)
         # avoid possible race if we already moved to a new app when
         # the reviews become ready
         # (we only check for pkgname currently to avoid breaking on
         #  software-center totem)
-        if self.app.pkgname != app.pkgname:
+        if self.app is None or self.app.pkgname != app.pkgname:
             return
 
         # Start fetching relaxed reviews if we retrieved no data
@@ -998,7 +1007,7 @@ class AppDetailsView(Viewport):
             self._reviews_relaxed = True
             self._reviews_server_page = 1
             self.review_loader.get_reviews(
-                self.app, self._reviews_ready_callback,
+                self.app,
                 page=self._reviews_server_page,
                 language=self._reviews_server_language,
                 sort=self._review_sort_method,
@@ -1025,27 +1034,22 @@ class AppDetailsView(Viewport):
             # update global stats cache as well
             self.review_loader.update_review_stats(app, stats)
 
-        if my_votes:
-            self.reviews.update_useful_votes(my_votes)
+        curr_list = set(self.reviews.get_all_review_ids())
+        retrieved_a_new_review = False
+        for review in reviews_data:
+            if not review.id in curr_list:
+                retrieved_a_new_review = True
+                self.reviews.add_review(review)
 
-        if action:
-            self._review_update_single(action, single_review)
-        else:
-            curr_list = set(self.reviews.get_all_review_ids())
-            retrieved_a_new_review = False
-            for review in reviews_data:
-                if not review.id in curr_list:
-                    retrieved_a_new_review = True
-                    self.reviews.add_review(review)
-            if reviews_data and not retrieved_a_new_review:
-                # We retrieved data, but nothing new.  Keep going.
-                self._reviews_server_page += 1
-                self.review_loader.get_reviews(
-                    self.app, self._reviews_ready_callback,
-                    page=self._reviews_server_page,
-                    language=self._reviews_server_language,
-                    sort=self._review_sort_method,
-                    relaxed=self._reviews_relaxed)
+        if reviews_data and not retrieved_a_new_review:
+            # We retrieved data, but nothing new.  Keep going.
+            self._reviews_server_page += 1
+            self.review_loader.get_reviews(
+                self.app,
+                page=self._reviews_server_page,
+                language=self._reviews_server_language,
+                sort=self._review_sort_method,
+                relaxed=self._reviews_relaxed)
 
         self.reviews.configure_reviews_ui()
 
@@ -1165,9 +1169,9 @@ class AppDetailsView(Viewport):
         self.review_stats_widget = StarRatingsWidget()
         self.review_stats = Gtk.HBox()
         vb_inner.pack_start(
-            self.review_stats, False, False, 0)
+            self.review_stats, False, False, StockEms.SMALL)
         self.review_stats.pack_start(
-            self.review_stats_widget, False, False, StockEms.SMALL)
+            self.review_stats_widget, False, False, 0)
 
         #~ vb_inner.set_property("can-focus", True)
         self.title.a11y = vb_inner.get_accessible()
@@ -1261,23 +1265,12 @@ class AppDetailsView(Viewport):
         info_vb.pack_start(self.addon_view, False, False, 0)
 
         self.addons_statusbar = self.addons_manager.status_bar
-        self.addon_view.pack_start(self.addons_statusbar, False, False, 0)
+        self.addon_view.pack_end(self.addons_statusbar, False, False, 0)
         self.addon_view.connect('table-built', self._on_addon_table_built)
 
         self.addons_hbar = self._hbars[1]
         info_vb.pack_start(self.addons_hbar, False, False, StockEms.SMALL)
 
-        # recommendations
-        catview = CategoriesViewGtk(
-            self.datadir, None, self.cache, self.db, self.icons, None)
-        self.recommended_for_app_panel = RecommendationsPanelDetails(catview)
-        self.recommended_for_app_panel.connect(
-            "application-activated",
-            self._on_recommended_application_activated)
-        self.recommended_for_app_panel.show_all()
-        self.info_vb.pack_start(self.recommended_for_app_panel, False,
-            False, 0)
-
         # package info
         self.info_keys = []
 
@@ -1306,6 +1299,17 @@ class AppDetailsView(Viewport):
 
         vb.pack_start(self._hbars[2], False, False, 0)
 
+        # recommendations
+        self.recommended_for_app_panel = RecommendationsPanelDetails(
+                self.db,
+                self.properties_helper)
+        self.recommended_for_app_panel.connect(
+            "application-activated",
+            self._on_recommended_application_activated)
+        self.recommended_for_app_panel.show_all()
+        self.info_vb.pack_start(self.recommended_for_app_panel, False,
+            False, 0)
+
         # reviews cascade
         self.reviews.connect("new-review", self._on_review_new)
         self.reviews.connect("report-abuse", self._on_review_report_abuse)
@@ -1502,8 +1506,7 @@ class AppDetailsView(Viewport):
             self.addons_manager.configure(app_details.pkgname)
 
         # Update total size label
-        self.totalsize_info.set_value(_("Calculating..."))
-        GObject.timeout_add(500, self.update_totalsize)
+        self.update_totalsize()
 
         # Update addons state bar
         self.addons_statusbar.configure()
@@ -1551,9 +1554,7 @@ class AppDetailsView(Viewport):
             self.addon_view.hide()
             if self.addon_view.get_parent():
                 self.info_vb.remove(self.addon_view)
-            self.totalsize_info.set_value(_("Calculating..."))
-            GObject.idle_add(self.update_totalsize,
-                             priority=GObject.PRIORITY_LOW)
+            self.update_totalsize()
         self._update_recommendations(app_details.pkgname)
         self._update_reviews(app_details)
 
@@ -1610,6 +1611,15 @@ class AppDetailsView(Viewport):
         # display launcher location
         label = Gtk.Label(label=_("Find it in the menu: "))
         self.installed_where_hbox.pack_start(label, False, False, 0)
+        if is_gnome_shell_running():
+            class ActivitiesPseudoItem(object):
+                def get_icon(self):
+                    return ""
+
+                def get_name(self):
+                    return _("Activities")
+
+            where.insert(0, ActivitiesPseudoItem())
         for (i, item) in enumerate(where):
             icon = None
             iconname = None
@@ -1671,7 +1681,8 @@ class AppDetailsView(Viewport):
         self.installed_where_hbox.set_property("can-focus", False)
         self.installed_where_hbox.a11y.set_name('')
 
-        # display where-is-it for non-Unity configurations only
+        # exit here early if unity is running (but still
+        # show commandline args)
         if is_unity_running():
             # but still display available commands, even in unity
             # because these are not easily discoverable and we don't
@@ -1723,9 +1734,6 @@ class AppDetailsView(Viewport):
                     force = True
         self.pkg_state = self.app_details.pkg_state
 
-        # for compat with the base class
-        self.appdetails = self.app_details
-
         # update content
         # layout page
         if same_app and not force:
@@ -1783,36 +1791,32 @@ class AppDetailsView(Viewport):
         #parent_xid = get_parent_xid(self)
         self.reviews.new_review.disable()
         self.review_loader.spawn_write_new_review_ui(
-            self.app, version, self.appdetails.icon, origin,
-            parent_xid, self.datadir,
-            self._reviews_ready_callback,
-            done_callback=self._submit_reviews_done_callback)
+            self.app, version, self.app_details.icon, origin,
+            parent_xid, self.datadir)
 
     def _review_report_abuse(self, review_id):
         parent_xid = ''
         #parent_xid = get_parent_xid(self)
         self.review_loader.spawn_report_abuse_ui(
-            review_id, parent_xid, self.datadir, self._reviews_ready_callback)
+            review_id, parent_xid, self.datadir)
 
     def _review_submit_usefulness(self, review_id, is_useful):
         parent_xid = ''
         #parent_xid = get_parent_xid(self)
         self.review_loader.spawn_submit_usefulness_ui(
-            review_id, is_useful, parent_xid, self.datadir,
-            self._reviews_ready_callback)
+            review_id, is_useful, parent_xid, self.datadir)
 
     def _review_modify(self, review_id):
         parent_xid = ''
         #parent_xid = get_parent_xid(self)
         self.review_loader.spawn_modify_review_ui(
-            parent_xid, self.appdetails.icon, self.datadir, review_id,
-            self._reviews_ready_callback)
+            parent_xid, self.app_details.icon, self.datadir, review_id)
 
     def _review_delete(self, review_id):
         parent_xid = ''
         #parent_xid = get_parent_xid(self)
         self.review_loader.spawn_delete_review_ui(
-            review_id, parent_xid, self.datadir, self._reviews_ready_callback)
+            review_id, parent_xid, self.datadir)
 
     # internal callbacks
     def _on_cache_ready(self, cache):
@@ -2000,18 +2004,28 @@ class AppDetailsView(Viewport):
         if not self.totalsize_info.get_property('visible'):
             return False
 
+        self.totalsize_info.set_value(_("Calculating..."))
+
         while Gtk.events_pending():
             Gtk.main_iteration()
 
-        label_string = ""
-
-        res = self.cache.get_total_size_on_install(
+        self.cache.query_total_size_on_install(
             self.app_details.pkgname,
             self.addons_manager.addons_to_install,
             self.addons_manager.addons_to_remove,
             self.app.archive_suite)
-        total_download_size, total_install_size = res
-        if res == (0, 0) and isinstance(self.app, DebFileApplication):
+
+    def _on_query_total_size_on_install_done(self,
+                                             pkginfo,
+                                             pkgname,
+                                             total_download_size,
+                                             total_install_size):
+        if not self.app or self.app.pkgname != pkgname:
+            return
+        label_string = ""
+        if (total_download_size == 0 and
+            total_install_size == 0 and
+            isinstance(self.app, DebFileApplication)):
             total_install_size = self.app_details.installed_size
         if total_download_size > 0:
             download_size = GLib.format_size(total_download_size)
@@ -2020,6 +2034,7 @@ class AppDetailsView(Viewport):
             install_size = GLib.format_size(total_install_size)
             label_string += _("%s when installed") % (install_size)
         elif (total_install_size == 0 and
+              self.app_details and
               self.app_details.pkg_state == PkgStates.INSTALLED and
               not self.addons_manager.addons_to_install and
               not self.addons_manager.addons_to_remove):
@@ -2041,76 +2056,3 @@ class AppDetailsView(Viewport):
 
     def set_section(self, section):
         self.section = section
-
-
-def get_test_window_appdetails(pkgname="totem"):
-
-    from softwarecenter.db.pkginfo import get_pkg_info
-    cache = get_pkg_info()
-    cache.open()
-
-    from softwarecenter.db.database import StoreDatabase
-    xapian_base_path = "/var/cache/software-center"
-    pathname = os.path.join(xapian_base_path, "xapian")
-    db = StoreDatabase(pathname, cache)
-    db.open()
-
-    import softwarecenter.paths
-    datadir = softwarecenter.paths.datadir
-
-    from softwarecenter.ui.gtk3.utils import get_sc_icon_theme
-    icons = get_sc_icon_theme(datadir)
-
-    import softwarecenter.distro
-    distro = softwarecenter.distro.get_distro()
-
-    # gui
-    win = Gtk.Window()
-    scroll = Gtk.ScrolledWindow()
-    view = AppDetailsView(db, distro, icons, cache, datadir)
-
-    view.show_app(Application("", pkgname))
-    #view.show_app(Application("Pay App Example", "pay-app"))
-    #view.show_app(Application("3D Chess", "3dchess"))
-    #view.show_app(Application("Movie Player", "totem"))
-    #view.show_app(Application("ACE", "unace"))
-    #~ view.show_app(Application("", "apt"))
-
-    #view.show_app("AMOR")
-    #view.show_app("Configuration Editor")
-    #view.show_app("Artha")
-    #view.show_app("cournol")
-    #view.show_app("Qlix")
-
-    scroll.add(view)
-    scroll.show()
-    win.add(scroll)
-    win.set_size_request(600, 800)
-    win.show()
-    win.connect('destroy', Gtk.main_quit)
-    win.set_data("view", view)
-    return win
-
-
-if __name__ == "__main__":
-    def _show_app(view):
-        if view.app.pkgname == "totem":
-            view.show_app(Application("Pithos", "pithos"))
-        else:
-            view.show_app(Application("Movie Player", "totem"))
-        return True
-
-    import sys
-    if len(sys.argv) > 1:
-        pkgname = sys.argv[1]
-    else:
-        pkgname = "totem"
-
-    win = get_test_window_appdetails(pkgname=pkgname)
-
-    # keep it spinning to test for re-draw issues and memleaks
-    #view = win.get_data("view")
-    #GObject.timeout_add_seconds(2, _show_app, view)
-
-    # run it
-    Gtk.main()
