@@ -21,22 +21,18 @@ from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Gdk
 
-import ConfigParser
 import logging
 import os
 import json
-import sys
-import urllib
 
 from gi.repository import WebKit as webkit
 
 from gettext import gettext as _
 
-from softwarecenter.backend import get_install_backend
+from softwarecenter.backend.installbackend import get_install_backend
 from softwarecenter.ui.gtk3.dialogs import show_accept_tos_dialog
 from softwarecenter.config import get_config
 from softwarecenter.ui.gtk3.utils import get_parent
-from softwarecenter.ui.gtk3.views.webkit import ScrolledWebkitWindow
 
 LOG = logging.getLogger(__name__)
 
@@ -103,26 +99,35 @@ center no-repeat;
     }
 
     def __init__(self):
-        GObject.GObject.__init__(self)
+        super(PurchaseView, self).__init__()
         self.wk = None
         self._wk_handlers_blocked = False
         self._oauth_token = None
         self.config = get_config()
 
+    def _log_debug_output(self, *args):
+        LOG.info("uri changed: '%s'", self.wk.webkit.get_property("uri"))
+
     def init_view(self):
         if self.wk is None:
+            # delaying this import make the window appear on the raspi
+            # 3s quicker
+            from softwarecenter.ui.gtk3.widgets.webkit import (
+                ScrolledWebkitWindow)
             self.wk = ScrolledWebkitWindow()
+            self.pack_start(self.wk, True, True, 0)
             # automatically fill in the email in the login page
-            email = ""
-            if self.config.has_option("general", "email"):
-                email = self.config.get("general", "email")
-            self.wk.webkit.set_auto_insert_email(email)
+            self.wk.webkit.set_auto_insert_email(self.config.email)
             #self.wk.webkit.connect("new-window-policy-decision-requested",
             #    self._on_new_window)
             self.wk.webkit.connect("create-web-view", self._on_create_web_view)
             self.wk.webkit.connect("close-web-view", self._on_close_web_view)
             self.wk.webkit.connect("console-message", self._on_console_message)
 
+            # check if the user wants url debugging
+            if os.environ.get("SOFTWARE_CENTER_DEBUG_WEBKIT"):
+                self.wk.webkit.connect("notify::uri", self._log_debug_output)
+
             # a possible way to do IPC (script or title change)
             self.wk.webkit.connect("script-alert", self._on_script_alert)
             self.wk.webkit.connect("title-changed", self._on_title_changed)
@@ -134,16 +139,13 @@ center no-repeat;
         self._unblock_wk_handlers()
 
     def _ask_for_tos_acceptance_if_needed(self):
-        try:
-            accepted_tos = self.config.getboolean("general", "accepted_tos")
-        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
-            accepted_tos = False
+        accepted_tos = self.config.user_accepted_tos
         if not accepted_tos:
             # show the dialog and ensure the user accepts it
             res = show_accept_tos_dialog(get_parent(self))
             if not res:
                 return False
-            self.config.set("general", "accepted_tos", "yes")
+            self.config.user_accepted_tos = True
             return True
         return True
 
@@ -161,15 +163,15 @@ center no-repeat;
         self.iconname = iconname
         self.wk.webkit.load_html_string(self.LOADING_HTML, "file:///")
         self.wk.show()
-        while Gtk.events_pending():
-            Gtk.main_iteration()
+        context = GObject.main_context_default()
+        while context.pending():
+            context.iteration()
         if url:
             self.wk.webkit.load_uri(url)
         elif html:
             self.wk.webkit.load_html_string(html, "file:///")
         else:
             self.wk.webkit.load_html_string(DUMMY_HTML, "file:///")
-        self.pack_start(self.wk, True, True, 0)
         # only for debugging
         if os.environ.get("SOFTWARE_CENTER_DEBUG_BUY"):
             GObject.timeout_add_seconds(1, _generate_events, self)
@@ -182,11 +184,13 @@ center no-repeat;
         return True
 
     def _on_close_web_view(self, view):
-        win = view.get_data("win")
+        win = view.parent_win
         win.destroy()
         return True
 
     def _on_create_web_view(self, view, frame):
+        from softwarecenter.ui.gtk3.widgets.webkit import (
+                ScrolledWebkitWindow)
         win = Gtk.Window()
         win.set_size_request(400, 400)
         wk = ScrolledWebkitWindow(include_progress_ui=True)
@@ -194,7 +198,7 @@ center no-repeat;
         win.add(wk)
         win.show_all()
         # make sure close will work later
-        wk.webkit.set_data("win", win)
+        wk.webkit.parent_win = win
         # find and set parent
         w = self.wk.get_parent()
         while w.get_parent():
@@ -251,7 +255,7 @@ center no-repeat;
         except:
             LOG.debug("error processing json: '%s'" % json_string)
             return
-        if res["successful"] == False:
+        if res["successful"] is False:
             if (res.get("user_canceled", False) or
                 # note the different spelling
                 res.get("user_cancelled", False) or
@@ -400,44 +404,3 @@ def _generate_events(view):
 #     # for debugging only
 #    def _on_key_press(dialog, event):
 #        print event, event.keyval
-
-
-def get_test_window_purchaseview():
-    #url = "http://www.animiertegifs.de/java-scripts/alertbox.php"
-    url = "http://www.ubuntu.cohtml=DUMMY_m"
-    #d = PurchaseDialog(app=None, url="http://spiegel.de")
-    from softwarecenter.enums import BUY_SOMETHING_HOST
-    url = BUY_SOMETHING_HOST + "/subscriptions/en/ubuntu/maverick/+new/?%s" % (
-        urllib.urlencode({
-                'archive_id': "mvo/private-test",
-                'arch': "i386",
-                }))
-    # use cmdline if available
-    if len(sys.argv) > 1:
-        url = sys.argv[1]
-    # useful for debugging
-    #d.connect("key-press-event", _on_key_press)
-    #GObject.timeout_add_seconds(1, _generate_events, d)
-
-    widget = PurchaseView()
-    from mock import Mock
-    widget.config = Mock()
-
-    win = Gtk.Window()
-    win.set_data("view", widget)
-    win.add(widget)
-    win.set_size_request(600, 500)
-    win.set_position(Gtk.WindowPosition.CENTER)
-    win.show_all()
-    win.connect('destroy', Gtk.main_quit)
-
-    widget.initiate_purchase(app=None, iconname=None, url=url)
-    #widget.initiate_purchase(app=None, iconname=None, html=DUMMY_HTML)
-
-    return win
-
-if __name__ == "__main__":
-    import softwarecenter.paths
-    softwarecenter.paths.datadir = "./data"
-    win = get_test_window_purchaseview()
-    Gtk.main()
