@@ -21,8 +21,7 @@ import cairo
 from gi.repository import Gtk, Gdk, Pango, GObject, GdkPixbuf
 from gettext import gettext as _
 
-from softwarecenter.backend import get_install_backend
-from softwarecenter.db.application import AppDetails
+from softwarecenter.backend.installbackend import get_install_backend
 from softwarecenter.enums import Icons
 from softwarecenter.ui.gtk3.em import StockEms, em
 from softwarecenter.ui.gtk3.drawing import darken
@@ -38,6 +37,8 @@ def _update_icon(image, icon, icon_size):
         image = image.set_from_pixbuf(icon.get_pixbuf())
     elif isinstance(icon, str):
         image = image.set_from_icon_name(icon, icon_size)
+    elif icon is None:
+        image = Gtk.Image()
     else:
         msg = "Acceptable icon values: None, GdkPixbuf, GtkImage or str"
         raise TypeError(msg)
@@ -107,12 +108,10 @@ class LabelTile(TileButton):
                              cr,
                              3, 3,
                              A.width - 6, A.height - 6)
-
+        cr.restore()
         for child in self:
             self.propagate_draw(child, cr)
 
-        cr.restore()
-
     def on_enter(self, widget, event):
         window = self.get_window()
         window.set_cursor(_HAND)
@@ -148,12 +147,10 @@ class CategoryTile(TileButton):
                              cr,
                              3, 3,
                              A.width - 6, A.height - 6)
-
+        cr.restore()
         for child in self:
             self.propagate_draw(child, cr)
 
-        cr.restore()
-
     def on_enter(self, widget, event):
         window = self.get_window()
         window.set_cursor(_HAND)
@@ -178,6 +175,7 @@ class FeaturedTile(TileButton):
         label = helper.get_appname(doc)
         icon = helper.get_icon_at_size(doc, icon_size, icon_size)
         stats = helper.get_review_stats(doc)
+
         self.helper = helper
         helper.update_availability(doc)
         helper.connect("needs-refresh", self._on_needs_refresh, doc, icon_size)
@@ -247,15 +245,8 @@ class FeaturedTile(TileButton):
             _global_featured_tile_width = max(_global_featured_tile_width,
                                               req_width)
 
-        details = AppDetails(db=helper.db, doc=doc)
         # TRANSLATORS: Free here means Gratis
-        price = details.price or _("Free")
-        if price == '0.00':
-            # TRANSLATORS: Free here means Gratis
-            price = _("Free")
-        # TRANSLATORS: Free here means Gratis
-        if price != _("Free"):
-            price = 'US$ ' + price
+        price = helper.get_display_price(doc)
         self.price = Gtk.Label.new(
             '<span font_desc="%i">%s</span>' % (em(0.6), price))
         self.price.set_use_markup(True)
@@ -271,9 +262,8 @@ class FeaturedTile(TileButton):
 
         self.backend = get_install_backend()
         self.backend.connect("transaction-finished",
-                             self.on_transaction_finished,
-                             helper, doc)
-
+                        self.on_transaction_finished,
+                        helper, doc)
         self.connect("enter-notify-event", self.on_enter)
         self.connect("leave-notify-event", self.on_leave)
         self.connect("button-press-event", self.on_press)
@@ -306,9 +296,6 @@ class FeaturedTile(TileButton):
                              3, 3,
                              A.width - 6, A.height - 6)
 
-        for child in self:
-            self.propagate_draw(child, cr)
-
         if self.is_installed:
             # paint installed tick overlay
             if self.get_direction() != Gtk.TextDirection.RTL:
@@ -319,8 +306,9 @@ class FeaturedTile(TileButton):
 
             Gdk.cairo_set_source_pixbuf(cr, self._overlay, x, y)
             cr.paint()
-
         cr.restore()
+        for child in self:
+            self.propagate_draw(child, cr)
 
     def on_transaction_finished(self, backend, result, helper, doc):
         trans_pkgname = str(result.pkgname)
@@ -479,6 +467,8 @@ class SectionSelector(TileToggleButton):
 
         self.connect('size-allocate', self.on_size_allocate)
         self.connect('style-updated', self.on_style_updated)
+        # workaround broken engines (LP: #1021308)
+        self.emit("style-updated")
 
     def on_size_allocate(self, *args):
         alloc = self.get_allocation()
@@ -533,6 +523,7 @@ class SectionSelector(TileToggleButton):
         self._bg_cache[state] = surf
 
     def do_draw(self, cr):
+        cr.save()
         state = self.get_state_flags()
         if self.get_active():
             if state not in self._bg_cache:
@@ -540,7 +531,7 @@ class SectionSelector(TileToggleButton):
 
             cr.set_source_surface(self._bg_cache[state], 0, 0)
             cr.paint()
-
+        cr.restore()
         for child in self:
             self.propagate_draw(child, cr)
 
@@ -624,43 +615,3 @@ class MoreLink(Gtk.Button):
     def on_leave(self, widget, event):
         window = self.get_window()
         window.set_cursor(None)
-
-
-def _build_channels_list(popup):
-    for i in range(3):
-        item = Gtk.MenuItem.new()
-        label = Gtk.Label.new("channel_name %s" % i)
-        box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, StockEms.MEDIUM)
-        box.pack_start(label, False, False, 0)
-        item.add(box)
-        item.show_all()
-        popup.attach(item, 0, 1, i, i + 1)
-
-
-def get_test_buttons_window():
-    win = Gtk.Window()
-    win.set_size_request(200, 200)
-
-    vb = Gtk.VBox(spacing=12)
-    win.add(vb)
-
-    link = Link("<small>test link</small>", uri="www.google.co.nz")
-    vb.pack_start(link, False, False, 0)
-
-    button = Gtk.Button()
-    button.set_label("channels")
-    channels_button = ChannelSelector(button)
-    channels_button.parent_style_type = Gtk.Window
-    channels_button.set_build_func(_build_channels_list)
-    hb = Gtk.HBox()
-    hb.pack_start(button, False, False, 0)
-    hb.pack_start(channels_button, False, False, 0)
-    vb.pack_start(hb, False, False, 0)
-
-    win.show_all()
-    win.connect("destroy", Gtk.main_quit)
-    return win
-
-if __name__ == "__main__":
-    win = get_test_buttons_window()
-    Gtk.main()
