@@ -72,6 +72,9 @@ class CellRendererAppView(Gtk.CellRendererText):
         self.selected_height = 0
         self.show_ratings = show_ratings
 
+        self.icon_x_offset = 0
+        self.icon_y_offset = 0
+
         # button packing
         self.button_spacing = 0
         self._buttons = {
@@ -120,7 +123,7 @@ class CellRendererAppView(Gtk.CellRendererText):
 
     def _render_price(self, context, cr, app, layout, cell_area, xpad, ypad,
         is_rtl):
-        layout.set_markup("US$ %s" % self.model.get_price(app), -1)
+        layout.set_markup("%s" % self.model.get_display_price(app))
 
         if is_rtl:
             x = cell_area.x + xpad
