@@ -4,12 +4,14 @@ from math import pi as PI
 PI_OVER_180 = PI / 180
 import softwarecenter.paths
 
-from gi.repository import Gtk, Gdk
+from gi.repository import Gtk, Gdk, GObject
 
 from buttons import MoreLink
 from softwarecenter.ui.gtk3.em import StockEms
 from softwarecenter.ui.gtk3.drawing import rounded_rect
-from softwarecenter.ui.gtk3.widgets.spinner import SpinnerNotebook
+from softwarecenter.ui.gtk3.widgets.spinner import (SpinnerView,
+                                                    SpinnerNotebook)
+from softwarecenter.ui.gtk3.widgets.buttons import FeaturedTile
 
 
 class FlowableGrid(Gtk.Fixed):
@@ -80,7 +82,7 @@ class FlowableGrid(Gtk.Fixed):
 
     # overrides
     def do_get_request_mode(self):
-        return Gtk.SizeRequestMode.HEIGHT_FOR_WIDTH
+        return Gtk.SizeRequestMode.WIDTH_FOR_HEIGHT
 
     def do_get_preferred_height_for_width(self, width):
         old = self.get_allocation()
@@ -187,6 +189,39 @@ class FlowableGrid(Gtk.Fixed):
                 del child
 
 
+class TileGrid(FlowableGrid):
+    ''' a flowable layout widget that holds a collection of TileButtons
+    '''
+    __gsignals__ = {
+        "application-activated": (GObject.SignalFlags.RUN_LAST,
+                                  None,
+                                  (GObject.TYPE_PYOBJECT, ),
+                                 ),
+        }
+
+    def add_tiles(self, properties_helper, docs, amount):
+        '''Adds application tiles to an ApplicationTileGrid:
+                properties_help -- an instance of the PropertiesHelper object
+                docs -- xapian documents (apps)
+                amount -- number of tiles to add from start of doc range
+        '''
+        amount = min(len(docs), amount)
+        for doc in docs[0:amount]:
+            tile = FeaturedTile(properties_helper, doc)
+            tile.connect('clicked', self._on_app_clicked,
+                         properties_helper.get_application(doc))
+            self.add_child(tile)
+
+    # private
+    def _on_app_clicked(self, btn, app):
+        """emit signal when a tile is clicked"""
+        def timeout_emit():
+            self.emit("application-activated", app)
+            return False
+
+        GObject.timeout_add(50, timeout_emit)
+
+
 # first tier of caching, cache component assets from which frames are
 # rendered
 _frame_asset_cache = {}
@@ -201,10 +236,10 @@ class Frame(Gtk.Alignment):
 
     def __init__(self, padding=0):
         Gtk.Alignment.__init__(self)
-        # set padding + some additional padding in the bottom, left and
-        # right edges to factor in the dropshadow width, and ensure even
-        # visual border
-        self.set_padding(padding, padding + 2, padding + 1, padding + 1)
+        # set padding + some additional padding to factor in the
+        # dropshadow+border width
+        # padding is (top, bottom, left, right)
+        self.set_padding(padding, padding + 2, padding + 3, padding + 3)
 
         self._cache_art_assets()
         # second tier of caching, cache resultant surface of
@@ -213,6 +248,8 @@ class Frame(Gtk.Alignment):
         self._allocation = Gdk.Rectangle()
         self.connect("size-allocate", self.on_size_allocate)
         self.connect("style-updated", self.on_style_updated)
+        # workaround broken engines (LP: #1021308)
+        self.emit("style-updated")
 
     def on_style_updated(self, widget):
         self._frame_surface_cache = None
@@ -456,8 +493,12 @@ class FramedHeaderBox(FramedBox):
         self.content_box = Gtk.Box.new(orientation, spacing)
         self.content_box.show()
         # finally, a notebook for the spinner and the content box to share
-        self.spinner_notebook = SpinnerNotebook(self.content_box)
+        self.spinner_notebook = SpinnerNotebook(self.content_box,
+                                                spinner_size=SpinnerView.SMALL)
         self.box.add(self.spinner_notebook)
+        # make the "More" button, but don't add it to the header unless/until
+        # we get a header_implements_more_button
+        self.more = MoreLink()
 
     def on_draw(self, cr):
         a = self.get_allocation()
@@ -477,17 +518,6 @@ class FramedHeaderBox(FramedBox):
     def pack_end(self, *args, **kwargs):
         return self.content_box.pack_end(*args, **kwargs)
 
-    # XXX: non-functional with current code...
-    #~ def set_header_expand(self, expand):
-        #~ alignment = self.header_alignment
-        #~ if expand:
-            #~ expand = 1.0
-        #~ else:
-            #~ expand = 0.0
-        #~ alignment.set(alignment.get_property("xalign"),
-                      #~ alignment.get_property("yalign"),
-                      #~ expand, 1.0)
-
     def set_header_position(self, position):
         alignment = self.header_alignment
         alignment.set(position, 0.5,
@@ -506,18 +536,16 @@ class FramedHeaderBox(FramedBox):
         self.title.set_markup(self.MARKUP % label)
 
     def header_implements_more_button(self):
-        if not hasattr(self, "more"):
-            self.more = MoreLink()
+        if self.more not in self.header:
             self.header.pack_end(self.more, False, False, 0)
 
     def remove_more_button(self):
-        if hasattr(self, "more"):
+        if self.more in self.header:
             self.header.remove(self.more)
-            del self.more
 
     def render_header(self, cr, a, border_radius, assets):
 
-        if hasattr(self, "more"):
+        if self.more in self.header:
             context = self.get_style_context()
 
             # set the arrow fill color
@@ -579,30 +607,3 @@ class FramedHeaderBox(FramedBox):
         # paint the containers children
         for child in self:
             self.propagate_draw(child, cr)
-
-
-# this is used in the automatic tests
-def get_test_container_window():
-    win = Gtk.Window()
-    win.set_size_request(500, 300)
-    f = FlowableGrid()
-
-    import buttons
-
-    for i in range(10):
-        t = buttons.CategoryTile("test", "folder")
-        f.add_child(t)
-
-    scroll = Gtk.ScrolledWindow()
-    scroll.add_with_viewport(f)
-
-    win.add(scroll)
-    win.show_all()
-
-    win.connect("destroy", lambda x: Gtk.main_quit())
-    return win
-
-if __name__ == '__main__':
-    win = get_test_container_window()
-    win.show_all()
-    Gtk.main()
