@@ -448,6 +448,8 @@ class TextBlock(Gtk.EventBox):
 
         self.connect("size-allocate", self.on_size_allocate)
         self.connect('style-updated', self._on_style_updated)
+        # workaround broken engines (LP: #1021308)
+        self.emit("style-updated")
 
     def on_size_allocate(self, *args):
         allocation = self.get_allocation()
@@ -1035,7 +1037,7 @@ class TextBlock(Gtk.EventBox):
     def render(self, widget, cr):
         if not self.order:
             return
-
+        cr.save()
         a = self.get_allocation()
         for layout in self.order:
             lx, ly = layout.get_position()
@@ -1067,6 +1069,7 @@ class TextBlock(Gtk.EventBox):
         # draw the cursor
         if self.PAINT_PRIMARY_CURSOR and self.has_focus():
             self.cursor.draw(cr, self._get_layout(self.cursor), a)
+        cr.restore()
 
     def append_paragraph(self, p, vspacing=None):
         l = self._new_layout()
@@ -1184,265 +1187,3 @@ class AppDescription(Gtk.VBox):
 
     def select_all(self):
         return TextBlock.select_all(self.description)
-
-
-def get_test_description_window():
-    EXAMPLE0 = """p7zip is the Unix port of 7-Zip, a file archiver that \
-archives with very high compression ratios.
-
-p7zip-full provides:
-
- - /usr/bin/7za a standalone version of the 7-zip tool that handles
-   7z archives (implementation of the LZMA compression algorithm) and some \
-other formats.
-
- - /usr/bin/7z not only does it handle 7z but also ZIP, Zip64, CAB, RAR, \
-ARJ, GZIP,
-   BZIP2, TAR, CPIO, RPM, ISO and DEB archives. 7z compression is 30-50% \
-better than ZIP compression.
-
-p7zip provides 7zr, a light version of 7za, and p7zip a gzip like wrapper \
-around 7zr."""
-
-    EXAMPLE1 = """Transmageddon supports almost any format as its input and \
-can generate a very large host of output files. The goal of the application \
-was to help people to create the files they need to be able to play on their \
-mobile devices and for people not hugely experienced with multimedia to \
-generate a multimedia file without having to resort to command line tools \
-with ungainly syntaxes.
-The currently supported codecs are:
- * Containers:
-  - Ogg
-  - Matroska
-  - AVI
-  - MPEG TS
-  - flv
-  - QuickTime
-  - MPEG4
-  - 3GPP
-  - MXT
- * Audio encoders:
-  - Vorbis
-  - FLAC
-  - MP3
-  - AAC
-  - AC3
-  - Speex
-  - Celt
- * Video encoders:
-  - Theora
-  - Dirac
-  - H264
-  - MPEG2
-  - MPEG4/DivX5
-  - xvid
-  - DNxHD
-It also provide the support for the GStreamer's plugins auto-search."""
-
-    EXAMPLE2 = """File-roller is an archive manager for the GNOME \
-environment. It allows you to:
- * Create and modify archives.
- * View the content of an archive.
- * View a file contained in an archive.
- * Extract files from the archive.
-File-roller supports the following formats:
- * Tar (.tar) archives, including those compressed with
-   gzip (.tar.gz, .tgz), bzip (.tar.bz, .tbz), bzip2 (.tar.bz2, .tbz2),
-   compress (.tar.Z, .taz), lzip (.tar.lz, .tlz), lzop (.tar.lzo, .tzo),
-   lzma (.tar.lzma) and xz (.tar.xz)
- * Zip archives (.zip)
- * Jar archives (.jar, .ear, .war)
- * 7z archives (.7z)
- * iso9660 CD images (.iso)
- * Lha archives (.lzh)
- * Single files compressed with gzip (.gz), bzip (.bz), bzip2 (.bz2),
-   compress (.Z), lzip (.lz), lzop (.lzo), lzma (.lzma) and xz (.xz)
-File-roller doesn't perform archive operations by itself, but relies on \
-standard tools for this."""
-
-    EXAMPLE3 = """This package includes the following CTAN packages:
- Asana-Math -- A font to typeset maths in Xe(La)TeX.
- albertus --
- allrunes -- Fonts and LaTeX package for almost all runes.
- antiqua -- the URW Antiqua Condensed Font.
- antp -- Antykwa Poltawskiego: a Type 1 family of Polish traditional type.
- antt -- Antykwa Torunska: a Type 1 family of a Polish traditional type.
- apl -- Fonts for typesetting APL programs.
- ar -- Capital A and capital R ligature for Apsect Ratio.
- archaic -- A collection of archaic fonts.
- arev -- Fonts and LaTeX support files for Arev Sans.
- ascii -- Support for IBM "standard ASCII" font.
- astro -- Astronomical (planetary) symbols.
- atqolive --
- augie -- Calligraphic font for typesetting handwriting.
- auncial-new -- Artificial Uncial font and LaTeX support macros.
- aurical -- Calligraphic fonts for use with LaTeX in T1 encoding.
- barcodes -- Fonts for making barcodes.
- bayer -- Herbert Bayers Universal Font For Metafont.
- bbding -- A symbol (dingbat) font and LaTeX macros for its use.
- bbm -- "Blackboard-style" cm fonts.
- bbm-macros -- LaTeX support for "blackboard-style" cm fonts.
- bbold -- Sans serif blackboard bold.
- belleek -- Free replacement for basic MathTime fonts.
- bera -- Bera fonts.
- blacklettert1 -- T1-encoded versions of Haralambous old German fonts.
- boisik -- A font inspired by Baskerville design.
- bookhands -- A collection of book-hand fonts.
- braille -- Support for braille.
- brushscr -- A handwriting script font.
- calligra -- Calligraphic font.
- carolmin-ps -- Adobe Type 1 format of Carolingian Minuscule fonts.
- cherokee -- A font for the Cherokee script.
- clarendo --
- cm-lgc -- Type 1 CM-based fonts for Latin, Greek and Cyrillic.
- cmbright -- Computer Modern Bright fonts.
- cmll -- Symbols for linear logic.
- cmpica -- A Computer Modern Pica variant.
- coronet --
- courier-scaled -- Provides a scaled Courier font.
- cryst -- Font for graphical symbols used in crystallography.
- cyklop -- The Cyclop typeface.
- dancers -- Font for Conan Doyle's "The Dancing Men".
- dice -- A font for die faces.
- dictsym -- DictSym font and macro package
- dingbat -- Two dingbat symbol fonts.
- doublestroke -- Typeset mathematical double stroke symbols.
- dozenal -- Typeset documents using base twelve numbering (also called
-  "dozenal")
- duerer -- Computer Duerer fonts.
- duerer-latex -- LaTeX support for the Duerer fonts.
- ean -- Macros for making EAN barcodes.
- ecc -- Sources for the European Concrete fonts.
- eco -- Oldstyle numerals using EC fonts.
- eiad -- Traditional style Irish fonts.
- eiad-ltx -- LaTeX support for the eiad font.
- elvish -- Fonts for typesetting Tolkien Elvish scripts.
- epigrafica -- A Greek and Latin font.
- epsdice -- A scalable dice "font".
- esvect -- Vector arrows.
- eulervm -- Euler virtual math fonts.
- euxm --
- feyn -- A font for in-text Feynman diagrams.
- fge -- A font for Frege's Grundgesetze der Arithmetik.
- foekfont -- The title font of the Mads Fok magazine.
- fonetika -- Support for the danish "Dania" phonetic system.
- fourier -- Using Utopia fonts in LaTeX documents.
- fouriernc -- Use New Century Schoolbook text with Fourier maths fonts.
- frcursive -- French cursive hand fonts.
- garamond --
- genealogy -- A compilation genealogy font.
- gfsartemisia -- A modern Greek font design.
- gfsbodoni -- A Greek and Latin font based on Bodoni.
- gfscomplutum -- A Greek font with a long history.
- gfsdidot -- A Greek font based on Didot's work.
- gfsneohellenic -- A Greek font in the Neo-Hellenic style.
- gfssolomos -- A Greek-alphabet font.
- gothic -- A collection of old German-style fonts.
- greenpoint -- The Green Point logo.
- groff --
- grotesq -- the URW Grotesk Bold Font.
- hands -- Pointing hand font.
- hfbright -- The hfbright fonts.
- hfoldsty -- Old style numerals with EC fonts.
- ifsym -- A collection of symbols.
- inconsolata -- A monospaced font, with support files for use with TeX.
- initials -- Adobe Type 1 decorative initial fonts.
- iwona -- A two-element sans-serif font.
- junicode -- A TrueType font for mediaevalists.
- kixfont -- A font for KIX codes.
- knuthotherfonts --
- kpfonts -- A complete set of fonts for text and mathematics.
- kurier -- A two-element sans-serif typeface.
- lettrgth --
- lfb -- A Greek font with normal and bold variants.
- libertine -- Use the font Libertine with LaTeX.
- libris -- Libris ADF fonts, with LaTeX support.
- linearA -- Linear A script fonts.
- logic -- A font for electronic logic design.
- lxfonts -- Set of slide fonts based on CM.
- ly1 -- Support for LY1 LaTeX encoding.
- marigold --
- mathabx -- Three series of mathematical symbols.
- mathdesign -- Mathematical fonts to fit with particular text fonts.
- mnsymbol -- Mathematical symbol font for Adobe MinionPro.
- nkarta -- A "new" version of the karta cartographic fonts.
- ocherokee -- LaTeX Support for the Cherokee language.
- ogham -- Fonts for typesetting Ogham script.
- oinuit -- LaTeX Support for the Inuktitut Language.
- optima --
- orkhun -- A font for orkhun script.
- osmanian -- Osmanian font for writing Somali.
- pacioli -- Fonts designed by Fra Luca de Pacioli in 1497.
- pclnfss -- Font support for current PCL printers.
- phaistos -- Disk of Phaistos font.
- phonetic -- MetaFont Phonetic fonts, based on Computer Modern.
- pigpen -- A font for the pigpen (or masonic) cipher.
- psafm --
- punk -- Donald Knuth's punk font.
- recycle -- A font providing the "recyclable" logo.
- sauter -- Wide range of design sizes for CM fonts.
- sauterfonts -- Use sauter fonts in LaTeX.
- semaphor -- Semaphore alphabet font.
- simpsons -- MetaFont source for Simpsons characters.
- skull -- A font to draw a skull.
- staves -- Typeset Icelandic staves and runic letters.
- tapir -- A simple geometrical font.
- tengwarscript -- LaTeX support for using Tengwar fonts.
- trajan -- Fonts from the Trajan column in Rome.
- umtypewriter -- Fonts to typeset with the xgreek package.
- univers --
- universa -- Herbert Bayer's 'universal' font.
- venturisadf -- Venturis ADF fonts collection.
- wsuipa -- International Phonetic Alphabet fonts.
- yfonts -- Support for old German fonts.
- zefonts -- Virtual fonts to provide T1 encoding from existing fonts."""
-
-    EXAMPLE4 = """Arista is a simple multimedia transcoder, it focuses on \
-being easy to use by making complex task of encoding for various devices \
-simple.
-Users should pick an input and a target device, choose a file to save to and \
-go. Features:
-* Presets for iPod, computer, DVD player, PSP, Playstation 3, and more.
-* Live preview to see encoded quality.
-* Automatically discover available DVD media and Video 4 Linux (v4l) devices.
-* Rip straight from DVD media easily (requires libdvdcss).
-* Rip straight from v4l devices.
-* Simple terminal client for scripting.
-* Automatic preset updating."""
-
-    def on_clicked(widget, desc_widget, descs):
-        widget.position += 1
-        if widget.position >= len(descs):
-            widget.position = 0
-        desc_widget.set_description(*descs[widget.position])
-
-    descs = ((EXAMPLE0, ''),
-             (EXAMPLE1, ''),
-             (EXAMPLE2, ''),
-             (EXAMPLE3, 'texlive-fonts-extra'),
-             (EXAMPLE4, ''))
-
-    win = Gtk.Window()
-    win.set_default_size(300, 400)
-    win.set_has_resize_grip(True)
-    vb = Gtk.VBox()
-    win.add(vb)
-    b = Gtk.Button('Next test description >>')
-    b.position = 0
-    vb.pack_start(b, False, False, 0)
-    scroll = Gtk.ScrolledWindow()
-    vb.add(scroll)
-    d = AppDescription()
-    #~ d.description.DEBUG_PAINT_BBOXES = True
-    d.set_description(EXAMPLE0, pkgname='')
-    scroll.add_with_viewport(d)
-    win.show_all()
-
-    b.connect("clicked", on_clicked, d, descs)
-    win.connect('destroy', lambda x: Gtk.main_quit())
-    return win
-
-if __name__ == '__main__':
-    win = get_test_description_window()
-    win.show_all()
-    Gtk.main()
