@@ -95,7 +95,7 @@ class UIReviewsList(Gtk.VBox):
     }
 
     def __init__(self, parent):
-        GObject.GObject.__init__(self)
+        Gtk.VBox.__init__(self)
         self.set_spacing(12)
         self.logged_in_person = get_person_from_config()
 
@@ -408,7 +408,7 @@ class UIReview(Gtk.VBox):
     def __init__(self, review_data=None, app_version=None,
                  logged_in_person=None, useful_votes=None,
                  first_for_version=True):
-        GObject.GObject.__init__(self)
+        Gtk.VBox.__init__(self)
         self.set_spacing(StockEms.SMALL)
 
         self.version_label = Gtk.Label()
@@ -713,7 +713,7 @@ class UIReview(Gtk.VBox):
             self.footer.pack_start(self.useful, False, False, 3)
             # add here, but only populate if its not the own review
             self.likebox = Gtk.HBox()
-            if already_voted == None and not current_user_reviewer:
+            if already_voted is None and not current_user_reviewer:
                 m = '<small>%s</small>'
                 self.yes_like = Link(m % _('Yes'))
                 self.yes_like.set_name("subtle-label")
@@ -755,7 +755,7 @@ class UIReview(Gtk.VBox):
         '''returns Gtk.Label() to be used as usefulness label depending
            on passed in parameters
         '''
-        if already_voted == None:
+        if already_voted is None:
             if useful_total == 0 and current_user_reviewer:
                 s = ""
             elif useful_total == 0:
@@ -943,46 +943,3 @@ class NoReviewYetWriteOne(EmbeddedMessage):
         msg = _('Be the first to contribute a review for this application')
 
         EmbeddedMessage.__init__(self, title, msg, 'text-editor')
-
-
-def get_test_reviews_window():
-    from mock import Mock
-
-    appdetails_mock = Mock()
-    appdetails_mock.version = "2.0"
-
-    parent = Mock()
-    parent.app_details = appdetails_mock
-
-    review_data = Mock()
-    review_data.app_name = "app"
-    review_data.usefulness_favorable = 10
-    review_data.usefulness_total = 12
-    review_data.usefulness_submit_error = False
-    review_data.reviewer_username = "name"
-    review_data.reviewer_displayname = "displayname"
-    review_data.date_created = "2011-01-01 18:00:00"
-    review_data.summary = "summary"
-    review_data.review_text = 10 * "loonng text"
-    review_data.rating = "3.0"
-    review_data.version = "1.0"
-
-    # create reviewslist
-    vb = UIReviewsList(parent)
-    vb.add_review(review_data)
-    vb.configure_reviews_ui()
-
-    win = Gtk.Window()
-    win.set_size_request(200, 200)
-    win.add(vb)
-    win.connect('destroy', Gtk.main_quit)
-    win.show_all()
-    return win
-
-if __name__ == "__main__":
-    import softwarecenter.paths
-    softwarecenter.paths.datadir = "./data"
-
-    win = get_test_reviews_window()
-
-    Gtk.main()
