@@ -17,6 +17,7 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import dbus
+import gc
 import gettext
 from gi.repository import GObject
 from gi.repository import Gio
@@ -66,20 +67,88 @@ class ExecutionTime(object):
     with ExecutinTime("db flush"):
         db.flush()
     """
-    def __init__(self, info="", with_traceback=False):
+    def __init__(self, info="", with_traceback=False,
+                 suppress_less_than_n_seconds=0.1):
         self.info = info
         self.with_traceback = with_traceback
+        self.suppress_less_than_n_seconds = suppress_less_than_n_seconds
 
     def __enter__(self):
         self.now = time.time()
 
     def __exit__(self, type, value, stack):
+        time_spend = time.time() - self.now
+        if time_spend < self.suppress_less_than_n_seconds:
+            return
         logger = logging.getLogger("softwarecenter.performance")
-        logger.debug("%s: %s" % (self.info, time.time() - self.now))
+        logger.debug("%s: %s" % (self.info, time_spend))
         if self.with_traceback:
             log_traceback("populate model from query: '%s' (threaded: %s)")
 
 
+class TraceActiveObjectTypes(object):
+    """ A context manager that tracks what and how many new object types
+        are added in the context block.
+
+    Note that due to the gc.collect() call this is slow and should not
+    be used in production code, but its very useful inside e.g. tests
+    """
+
+    def __init__(self, info):
+        self.info = info
+        self._known_obj_ids = set()
+
+    def __enter__(self):
+        gc.collect()
+        for obj in gc.get_objects():
+            self._known_obj_ids.add(id(obj))
+
+    def __exit__(self, atype, value, stack):
+        # ensure we start clean
+        gc.collect()
+        new_obj_types = {}
+        for obj in gc.get_objects():
+            if id(obj) in self._known_obj_ids:
+                continue
+            self._known_obj_ids.add(id(obj))
+            if not type(obj) in new_obj_types:
+                new_obj_types[type(obj)] = 0
+            new_obj_types[type(obj)] += 1
+        print "+++ new types after '%s':" % self.info
+        #print new_obj_types
+        for v in sorted(new_obj_types, key=new_obj_types.get):
+            print v, new_obj_types[v]
+        print "/+++\n"
+
+
+class TraceMemoryUsage(object):
+    """Trace amount of memory used by the app using /proc/$$/statm"""
+
+    def __init__(self, info):
+        self.info = info
+
+    def __enter__(self):
+        self.resident = self._get_mem_from_proc()["resident"]
+        self.data = self._get_mem_from_proc()["data"]
+
+    def __exit__(self, atype, value, stack):
+        now_resident = self._get_mem_from_proc()["resident"] - self.resident
+        now_data = self._get_mem_from_proc()["data"] - self.data
+        print "++++ MEMORY DELTA for '%s': res: %s data: %s (%s %s)\n" % (
+            self.info,
+            # assume page size of 4k here
+            get_nice_size(now_resident * 4 * 1024),
+            get_nice_size(now_data * 4 * 1024),
+            now_resident, now_data)
+
+    def _get_mem_from_proc(self):
+        with open("/proc/%i/statm" % os.getpid()) as f:
+            size, resident, share, text, lib, data, dt = f.readline().split()
+        return {'resident': int(resident),
+                'data': int(data),
+               }
+
+
 def utf8(s):
     """
     Takes a string or unicode object and returns a utf-8 encoded
@@ -334,7 +403,7 @@ def sources_filename_from_ppa_entry(entry):
     takes a PPA SourceEntry and returns a filename suitable for sources.list.d
     """
     import apt_pkg
-    name = "%s.list" % apt_pkg.URItoFileName(entry.uri)
+    name = "%s.list" % apt_pkg.uri_to_filename(entry.uri)
     return name
 
 
@@ -365,17 +434,25 @@ def release_filename_in_lists_from_deb_line(debline):
     return name
 
 
+def is_dbus_service_running(service_name):
+    running = False
+    try:
+        bus = dbus.SessionBus()
+        running = bus.name_has_owner(service_name)
+    except:
+        LOG.exception("could not check for dbus service %s" % service_name)
+    return running
+
+
+def is_gnome_shell_running():
+    return is_dbus_service_running("org.gnome.Shell")
+
+
 def is_unity_running():
     """
     return True if Unity is currently running
     """
-    unity_running = False
-    try:
-        bus = dbus.SessionBus()
-        unity_running = bus.name_has_owner("com.canonical.Unity")
-    except:
-        LOG.exception("could not check for Unity dbus service")
-    return unity_running
+    return is_dbus_service_running("com.canonical.Unity")
 
 
 def get_icon_from_theme(icons, iconname=None, iconsize=Icons.APP_ICON_SIZE,
@@ -388,7 +465,7 @@ def get_icon_from_theme(icons, iconname=None, iconsize=Icons.APP_ICON_SIZE,
     try:
         icon = icons.load_icon(iconname, iconsize, 0)
     except Exception as e:
-        LOG.warning(utf8("could not load icon '%s', displaying missing "\
+        LOG.warning(utf8("could not load icon '%s', displaying missing "
                          "icon instead: %s ") % (
                 utf8(iconname), utf8(e.message)))
         icon = icons.load_icon(missingicon, iconsize, 0)
@@ -532,8 +609,8 @@ def is_no_display_desktop_file(desktop_file):
 
 
 def get_nice_size(n_bytes):
-    nice_size = lambda s: [(s % 1024 ** i and "%.1f" % (s / 1024.0 ** i) or \
-        str(s / 1024 ** i)) + x.strip() for i, x in enumerate(' KMGTPEZY') \
+    nice_size = lambda s: [(s % 1024 ** i and "%.1f" % (s / 1024.0 ** i) or
+        str(s / 1024 ** i)) + x.strip() for i, x in enumerate(' KMGTPEZY')
         if s < 1024 ** (i + 1) or i == 8][0]
     return nice_size(n_bytes)
 
@@ -652,12 +729,12 @@ def mangle_paths_if_running_in_local_checkout():
         xapian_base_path = datadir
         # set new global datadir
         softwarecenter.paths.datadir = datadir
+        softwarecenter.paths.XAPIAN_BASE_PATH = xapian_base_path
         # also alter the app-install path
-        path = "%s/desktop/software-center.menu" % \
-            softwarecenter.paths.APP_INSTALL_PATH
-        if not os.path.exists(path):
+        path = "./build/share/app-install/desktop/software-center.menu"
+        if os.path.exists(path):
             softwarecenter.paths.APP_INSTALL_PATH = './build/share/app-install'
-            logging.warn("using local APP_INSTALL_PATH: %s" %\
+            logging.warn("using local APP_INSTALL_PATH: %s" %
                              softwarecenter.paths.APP_INSTALL_PATH)
     else:
         datadir = softwarecenter.paths.datadir
@@ -670,6 +747,13 @@ def get_uuid():
     return str(uuid.uuid4())
 
 
+def get_recommender_uuid():
+    """ the recommender service requires a UUID that does not contain
+        dashes
+    """
+    return get_uuid().replace("-", "")
+
+
 def get_lock(path):
     """ return a lock that can be released with release_lock on success
         and -1 on failure
